package sun.security.tools.keytool;

import java.util.EnumSet;
import java.security.spec.ECParameterSpec;
import sun.security.util.NamedCurve;
import java.security.interfaces.ECKey;
import sun.security.x509.AuthorityKeyIdentifierExtension;
import sun.security.x509.SubjectKeyIdentifierExtension;
import sun.security.x509.KeyIdentifier;
import sun.security.util.DerValue;
import sun.security.x509.SubjectInfoAccessExtension;
import sun.security.x509.AuthorityInfoAccessExtension;
import sun.security.x509.AccessDescription;
import sun.security.x509.SubjectAlternativeNameExtension;
import sun.security.x509.IssuerAlternativeNameExtension;
import sun.security.x509.ExtendedKeyUsageExtension;
import sun.security.x509.KeyUsageExtension;
import sun.security.x509.BasicConstraintsExtension;
import sun.security.x509.PKIXExtensions;
import sun.security.util.ObjectIdentifier;
import sun.security.x509.OIDName;
import sun.security.x509.IPAddressName;
import sun.security.x509.DNSName;
import sun.security.x509.RFC822Name;
import java.util.GregorianCalendar;
import java.security.Principal;
import java.util.Vector;
import java.util.Hashtable;
import java.security.MessageDigest;
import java.security.UnrecoverableEntryException;
import java.security.UnrecoverableKeyException;
import sun.misc.HexDumpEncoder;
import sun.security.x509.Extension;
import sun.security.util.KeyUtil;
import java.security.Timestamp;
import java.security.CodeSigner;
import java.security.cert.CertStore;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.security.cert.CertStoreException;
import java.security.cert.CertificateException;
import java.security.PublicKey;
import javax.security.auth.x500.X500Principal;
import sun.security.x509.GeneralNames;
import sun.security.x509.CRLDistributionPointsExtension;
import java.security.cert.X509CRL;
import sun.security.x509.URIName;
import sun.security.x509.GeneralName;
import sun.security.x509.DistributionPoint;
import java.security.cert.X509CRLSelector;
import sun.security.provider.certpath.CertStoreHelper;
import java.io.ByteArrayInputStream;
import java.net.URI;
import java.security.cert.CRL;
import java.util.Collection;
import java.util.Enumeration;
import java.security.KeyStoreException;
import java.nio.file.Files;
import java.nio.file.CopyOption;
import java.nio.file.Paths;
import java.security.Key;
import sun.security.util.SecurityProviderConstants;
import javax.crypto.SecretKey;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.SecretKeyFactory;
import javax.crypto.KeyGenerator;
import java.util.Locale;
import java.util.Base64;
import sun.security.x509.X509CRLImpl;
import sun.security.x509.CRLReasonCodeExtension;
import sun.security.x509.CRLExtensions;
import sun.security.x509.X509CRLEntryImpl;
import java.math.BigInteger;
import java.security.cert.X509CRLEntry;
import java.security.spec.PSSParameterSpec;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import sun.security.x509.CertificateExtensions;
import sun.security.pkcs.PKCS9Attribute;
import sun.security.pkcs10.PKCS10Attribute;
import sun.security.x509.CertificateX509Key;
import sun.security.pkcs10.PKCS10;
import sun.security.util.Pem;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import sun.security.x509.CertificateAlgorithmId;
import sun.security.x509.CertificateVersion;
import sun.security.x509.CertificateSerialNumber;
import java.util.Random;
import sun.security.util.SignatureUtil;
import sun.security.x509.AlgorithmId;
import java.security.Signature;
import java.security.PrivateKey;
import sun.security.x509.CertificateValidity;
import java.util.Date;
import sun.security.x509.X509CertImpl;
import sun.security.x509.X509CertInfo;
import sun.security.x509.X500Name;
import java.io.IOException;
import java.io.DataInputStream;
import java.util.Collections;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import sun.security.util.Password;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.security.Security;
import java.security.Provider;
import java.net.URLClassLoader;
import sun.security.tools.PathList;
import java.text.MessageFormat;
import sun.security.tools.KeyStoreUtil;
import java.util.Iterator;
import java.util.Arrays;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.text.Collator;
import java.util.ResourceBundle;
import java.security.CryptoPrimitive;
import sun.security.util.DisabledAlgorithmConstraints;
import java.util.List;
import java.security.cert.CertificateFactory;
import java.security.KeyStore;
import java.io.InputStream;
import java.io.File;
import java.util.Set;

public final class Main
{
    private static final byte[] CRLF;
    private boolean debug;
    private Main.Command command;
    private String sigAlgName;
    private String keyAlgName;
    private boolean verbose;
    private int keysize;
    private boolean rfc;
    private long validity;
    private String alias;
    private String dname;
    private String dest;
    private String filename;
    private String infilename;
    private String outfilename;
    private String srcksfname;
    private Set<Pair<String, String>> providers;
    private String storetype;
    private String srcProviderName;
    private String providerName;
    private String pathlist;
    private char[] storePass;
    private char[] storePassNew;
    private char[] keyPass;
    private char[] keyPassNew;
    private char[] newPass;
    private char[] destKeyPass;
    private char[] srckeyPass;
    private String ksfname;
    private File ksfile;
    private InputStream ksStream;
    private String sslserver;
    private String jarfile;
    private KeyStore keyStore;
    private boolean token;
    private boolean nullStream;
    private boolean kssave;
    private boolean noprompt;
    private boolean trustcacerts;
    private boolean nowarn;
    private boolean protectedPath;
    private boolean srcprotectedPath;
    private CertificateFactory cf;
    private KeyStore caks;
    private char[] srcstorePass;
    private String srcstoretype;
    private Set<char[]> passwords;
    private String startDate;
    private List<String> ids;
    private List<String> v3ext;
    private boolean inplaceImport;
    private String inplaceBackupName;
    private List<String> weakWarnings;
    private static final DisabledAlgorithmConstraints DISABLED_CHECK;
    private static final DisabledAlgorithmConstraints LEGACY_CHECK;
    private static final Set<CryptoPrimitive> SIG_PRIMITIVE_SET;
    private static final Class<?>[] PARAM_STRING;
    private static final String NONE = "NONE";
    private static final String P11KEYSTORE = "PKCS11";
    private static final String P12KEYSTORE = "PKCS12";
    private static final String keyAlias = "mykey";
    private static final ResourceBundle rb;
    private static final Collator collator;
    private static final String[] extSupported;
    
    private Main() {
        this.debug = false;
        this.command = null;
        this.sigAlgName = null;
        this.keyAlgName = null;
        this.verbose = false;
        this.keysize = -1;
        this.rfc = false;
        this.validity = 90L;
        this.alias = null;
        this.dname = null;
        this.dest = null;
        this.filename = null;
        this.infilename = null;
        this.outfilename = null;
        this.srcksfname = null;
        this.providers = null;
        this.storetype = null;
        this.srcProviderName = null;
        this.providerName = null;
        this.pathlist = null;
        this.storePass = null;
        this.storePassNew = null;
        this.keyPass = null;
        this.keyPassNew = null;
        this.newPass = null;
        this.destKeyPass = null;
        this.srckeyPass = null;
        this.ksfname = null;
        this.ksfile = null;
        this.ksStream = null;
        this.sslserver = null;
        this.jarfile = null;
        this.keyStore = null;
        this.token = false;
        this.nullStream = false;
        this.kssave = false;
        this.noprompt = false;
        this.trustcacerts = false;
        this.nowarn = false;
        this.protectedPath = false;
        this.srcprotectedPath = false;
        this.cf = null;
        this.caks = null;
        this.srcstorePass = null;
        this.srcstoretype = null;
        this.passwords = new HashSet();
        this.startDate = null;
        this.ids = new ArrayList();
        this.v3ext = new ArrayList();
        this.inplaceImport = false;
        this.inplaceBackupName = null;
        this.weakWarnings = new ArrayList();
    }
    
    public static void main(String[] array) throws Exception {
        new Main().run(array, System.out);
    }
    
    private void run(String[] array, PrintStream printStream) throws Exception {
        try {
            this.parseArgs(array);
            if (this.command != null) {
                this.doCommands(printStream);
            }
            this.printWeakWarnings(false);
            for (char[] a : this.passwords) {
                if (a != null) {
                    Arrays.fill(a, ' ');
                }
                else {
                    continue;
                }
            }
            if (this.ksStream != null) {
                this.ksStream.close();
            }
        }
        catch (Exception obj) {
            System.out.println(Main.rb.getString("keytool.error.") + obj);
            if (this.verbose) {
                obj.printStackTrace(System.out);
            }
            if (!this.debug) {
                System.exit(1);
                this.printWeakWarnings(false);
                for (char[] a2 : this.passwords) {
                    if (a2 != null) {
                        Arrays.fill(a2, ' ');
                    }
                    else {
                        continue;
                    }
                }
                if (this.ksStream != null) {
                    this.ksStream.close();
                }
            }
            else {
                throw obj;
            }
        }
        finally {
            this.printWeakWarnings(false);
            for (char[] a3 : this.passwords) {
                if (a3 != null) {
                    Arrays.fill(a3, ' ');
                }
                else {
                    continue;
                }
            }
            if (this.ksStream != null) {
                this.ksStream.close();
            }
        }
    }
    
    void parseArgs(String[] array) {
        int n = array.length == 0;
        int n2;
        for (n2 = 0; n2 < array.length && array[n2].startsWith("-"); ++n2) {
            String substring = array[n2];
            if (n2 == array.length - 1) {
                Main.Option[] values = Main.Option.values();
                int length = values.length;
                int i = 0;
                while (i < length) {
                    Main.Option option = values[i];
                    if (Main.collator.compare(substring, option.toString()) == 0) {
                        if (option.arg != null) {
                            this.errorNeedArgument(substring);
                            break;
                        }
                        break;
                    }
                    else {
                        ++i;
                    }
                }
            }
            String substring2 = null;
            int index = substring.indexOf(58);
            if (index > 0) {
                substring2 = substring.substring(index + 1);
                substring = substring.substring(0, index);
            }
            boolean n3 = false;
            Main.Command[] values2 = Main.Command.values();
            int length2 = values2.length;
            int j = 0;
            while (j < length2) {
                Main.Command command = values2[j];
                if (Main.collator.compare(substring, command.toString()) == 0) {
                    this.command = command;
                    n3 = true;
                    break;
                }
                ++j;
            }
            if (n3 == 0) {
                if (Main.collator.compare(substring, "-export") == 0) {
                    this.command = Main.Command.EXPORTCERT;
                }
                else if (Main.collator.compare(substring, "-genkey") == 0) {
                    this.command = Main.Command.GENKEYPAIR;
                }
                else if (Main.collator.compare(substring, "-import") == 0) {
                    this.command = Main.Command.IMPORTCERT;
                }
                else if (Main.collator.compare(substring, "-importpassword") == 0) {
                    this.command = Main.Command.IMPORTPASS;
                }
                else if (Main.collator.compare(substring, "-help") == 0) {
                    n = true;
                }
                else if (Main.collator.compare(substring, "-nowarn") == 0) {
                    this.nowarn = true;
                }
                else if (Main.collator.compare(substring, "-keystore") == 0 || Main.collator.compare(substring, "-destkeystore") == 0) {
                    this.ksfname = array[++n2];
                }
                else if (Main.collator.compare(substring, "-storepass") == 0 || Main.collator.compare(substring, "-deststorepass") == 0) {
                    this.storePass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.storePass);
                }
                else if (Main.collator.compare(substring, "-storetype") == 0 || Main.collator.compare(substring, "-deststoretype") == 0) {
                    this.storetype = KeyStoreUtil.niceStoreTypeName(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-srcstorepass") == 0) {
                    this.srcstorePass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.srcstorePass);
                }
                else if (Main.collator.compare(substring, "-srcstoretype") == 0) {
                    this.srcstoretype = KeyStoreUtil.niceStoreTypeName(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-srckeypass") == 0) {
                    this.srckeyPass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.srckeyPass);
                }
                else if (Main.collator.compare(substring, "-srcprovidername") == 0) {
                    this.srcProviderName = array[++n2];
                }
                else if (Main.collator.compare(substring, "-providername") == 0 || Main.collator.compare(substring, "-destprovidername") == 0) {
                    this.providerName = array[++n2];
                }
                else if (Main.collator.compare(substring, "-providerpath") == 0) {
                    this.pathlist = array[++n2];
                }
                else if (Main.collator.compare(substring, "-keypass") == 0) {
                    this.keyPass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.keyPass);
                }
                else if (Main.collator.compare(substring, "-new") == 0) {
                    this.newPass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.newPass);
                }
                else if (Main.collator.compare(substring, "-destkeypass") == 0) {
                    this.destKeyPass = this.getPass(substring2, array[++n2]);
                    this.passwords.add(this.destKeyPass);
                }
                else if (Main.collator.compare(substring, "-alias") == 0 || Main.collator.compare(substring, "-srcalias") == 0) {
                    this.alias = array[++n2];
                }
                else if (Main.collator.compare(substring, "-dest") == 0 || Main.collator.compare(substring, "-destalias") == 0) {
                    this.dest = array[++n2];
                }
                else if (Main.collator.compare(substring, "-dname") == 0) {
                    this.dname = array[++n2];
                }
                else if (Main.collator.compare(substring, "-keysize") == 0) {
                    this.keysize = Integer.parseInt(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-keyalg") == 0) {
                    this.keyAlgName = array[++n2];
                }
                else if (Main.collator.compare(substring, "-sigalg") == 0) {
                    this.sigAlgName = array[++n2];
                }
                else if (Main.collator.compare(substring, "-startdate") == 0) {
                    this.startDate = array[++n2];
                }
                else if (Main.collator.compare(substring, "-validity") == 0) {
                    this.validity = Long.parseLong(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-ext") == 0) {
                    this.v3ext.add(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-id") == 0) {
                    this.ids.add(array[++n2]);
                }
                else if (Main.collator.compare(substring, "-file") == 0) {
                    this.filename = array[++n2];
                }
                else if (Main.collator.compare(substring, "-infile") == 0) {
                    this.infilename = array[++n2];
                }
                else if (Main.collator.compare(substring, "-outfile") == 0) {
                    this.outfilename = array[++n2];
                }
                else if (Main.collator.compare(substring, "-sslserver") == 0) {
                    this.sslserver = array[++n2];
                }
                else if (Main.collator.compare(substring, "-jarfile") == 0) {
                    this.jarfile = array[++n2];
                }
                else if (Main.collator.compare(substring, "-srckeystore") == 0) {
                    this.srcksfname = array[++n2];
                }
                else if (Main.collator.compare(substring, "-provider") == 0 || Main.collator.compare(substring, "-providerclass") == 0) {
                    if (this.providers == null) {
                        this.providers = new HashSet(3);
                    }
                    String s = array[++n2];
                    String s2 = null;
                    if (array.length > n2 + 1) {
                        String s3 = array[n2 + 1];
                        if (Main.collator.compare(s3, "-providerarg") == 0) {
                            if (array.length == n2 + 2) {
                                this.errorNeedArgument(s3);
                            }
                            s2 = array[n2 + 2];
                            n2 += 2;
                        }
                    }
                    this.providers.add(Pair.of(s, s2));
                }
                else if (Main.collator.compare(substring, "-v") == 0) {
                    this.verbose = true;
                }
                else if (Main.collator.compare(substring, "-debug") == 0) {
                    this.debug = true;
                }
                else if (Main.collator.compare(substring, "-rfc") == 0) {
                    this.rfc = true;
                }
                else if (Main.collator.compare(substring, "-noprompt") == 0) {
                    this.noprompt = true;
                }
                else if (Main.collator.compare(substring, "-trustcacerts") == 0) {
                    this.trustcacerts = true;
                }
                else if (Main.collator.compare(substring, "-protected") == 0 || Main.collator.compare(substring, "-destprotected") == 0) {
                    this.protectedPath = true;
                }
                else if (Main.collator.compare(substring, "-srcprotected") == 0) {
                    this.srcprotectedPath = true;
                }
                else {
                    System.err.println(Main.rb.getString("Illegal.option.") + substring);
                    this.tinyHelp();
                }
            }
        }
        if (n2 < array.length) {
            System.err.println(Main.rb.getString("Illegal.option.") + array[n2]);
            this.tinyHelp();
        }
        if (this.command == null) {
            if (!(n == 0)) {
                this.usage();
            }
            else {
                System.err.println(Main.rb.getString("Usage.error.no.command.provided"));
                this.tinyHelp();
            }
        }
        else if (!(n == 0)) {
            this.usage();
            this.command = null;
        }
    }
    
    boolean isKeyStoreRelated(Main.Command command) {
        return command != Main.Command.PRINTCERT && command != Main.Command.PRINTCERTREQ;
    }
    
    void doCommands(PrintStream printStream) throws Exception {
        if ("PKCS11".equalsIgnoreCase(this.storetype) || KeyStoreUtil.isWindowsKeyStore(this.storetype)) {
            this.token = true;
            if (this.ksfname == null) {
                this.ksfname = "NONE";
            }
        }
        if ("NONE".equals(this.ksfname)) {
            this.nullStream = true;
        }
        if (this.token && !this.nullStream) {
            System.err.println(MessageFormat.format(Main.rb.getString(".keystore.must.be.NONE.if.storetype.is.{0}"), new Object[] { this.storetype }));
            System.err.println();
            this.tinyHelp();
        }
        if (this.token && (this.command == Main.Command.KEYPASSWD || this.command == Main.Command.STOREPASSWD)) {
            throw new UnsupportedOperationException(MessageFormat.format(Main.rb.getString(".storepasswd.and.keypasswd.commands.not.supported.if.storetype.is.{0}"), new Object[] { this.storetype }));
        }
        if (this.token && (this.keyPass != null || this.newPass != null || this.destKeyPass != null)) {
            throw new IllegalArgumentException(MessageFormat.format(Main.rb.getString(".keypass.and.new.can.not.be.specified.if.storetype.is.{0}"), new Object[] { this.storetype }));
        }
        if (this.protectedPath && (this.storePass != null || this.keyPass != null || this.newPass != null || this.destKeyPass != null)) {
            throw new IllegalArgumentException(Main.rb.getString("if.protected.is.specified.then.storepass.keypass.and.new.must.not.be.specified"));
        }
        if (this.srcprotectedPath && (this.srcstorePass != null || this.srckeyPass != null)) {
            throw new IllegalArgumentException(Main.rb.getString("if.srcprotected.is.specified.then.srcstorepass.and.srckeypass.must.not.be.specified"));
        }
        if (KeyStoreUtil.isWindowsKeyStore(this.storetype) && (this.storePass != null || this.keyPass != null || this.newPass != null || this.destKeyPass != null)) {
            throw new IllegalArgumentException(Main.rb.getString("if.keystore.is.not.password.protected.then.storepass.keypass.and.new.must.not.be.specified"));
        }
        if (KeyStoreUtil.isWindowsKeyStore(this.srcstoretype) && (this.srcstorePass != null || this.srckeyPass != null)) {
            throw new IllegalArgumentException(Main.rb.getString("if.source.keystore.is.not.password.protected.then.srcstorepass.and.srckeypass.must.not.be.specified"));
        }
        if (this.validity <= 0L) {
            throw new Exception(Main.rb.getString("Validity.must.be.greater.than.zero"));
        }
        if (this.providers != null) {
            URLClassLoader systemClassLoader;
            if (this.pathlist != null) {
                systemClassLoader = new URLClassLoader(PathList.pathToURLs(PathList.appendPath(PathList.appendPath(PathList.appendPath(null, System.getProperty("java.class.path")), System.getProperty("env.class.path")), this.pathlist)));
            }
            else {
                systemClassLoader = ClassLoader.getSystemClassLoader();
            }
            for (Pair pair : this.providers) {
                String s = (String)pair.fst;
                Class<Object> clazz;
                if (systemClassLoader != null) {
                    clazz = systemClassLoader.loadClass(s);
                }
                else {
                    clazz = Class.forName(s);
                }
                String s2 = (String)pair.snd;
                Object o;
                if (s2 == null) {
                    o = clazz.newInstance();
                }
                else {
                    o = clazz.getConstructor(Main.PARAM_STRING).newInstance(new Object[] { s2 });
                }
                if (!(o instanceof Provider)) {
                    throw new Exception(new MessageFormat(Main.rb.getString("provName.not.a.provider")).format(new Object[] { s }));
                }
                Security.addProvider((Provider)o);
            }
        }
        if (this.command == Main.Command.LIST && this.verbose && this.rfc) {
            System.err.println(Main.rb.getString("Must.not.specify.both.v.and.rfc.with.list.command"));
            this.tinyHelp();
        }
        if (this.command == Main.Command.GENKEYPAIR && this.keyPass != null && this.keyPass.length < 6) {
            throw new Exception(Main.rb.getString("Key.password.must.be.at.least.6.characters"));
        }
        if (this.newPass != null && this.newPass.length < 6) {
            throw new Exception(Main.rb.getString("New.password.must.be.at.least.6.characters"));
        }
        if (this.destKeyPass != null && this.destKeyPass.length < 6) {
            throw new Exception(Main.rb.getString("New.password.must.be.at.least.6.characters"));
        }
        if (this.ksfname == null) {
            this.ksfname = System.getProperty("user.home") + File.separator + ".keystore";
        }
        KeyStore keyStore = null;
        if (this.command == Main.Command.IMPORTKEYSTORE) {
            this.inplaceImport = this.inplaceImportCheck();
            if (this.inplaceImport) {
                keyStore = this.loadSourceKeyStore();
                if (this.storePass == null) {
                    this.storePass = this.srcstorePass;
                }
            }
        }
        if (this.isKeyStoreRelated(this.command) && !this.nullStream && !this.inplaceImport) {
            try {
                this.ksfile = new File(this.ksfname);
                if (this.ksfile.exists() && this.ksfile.length() == 0L) {
                    throw new Exception(Main.rb.getString("Keystore.file.exists.but.is.empty.") + this.ksfname);
                }
                this.ksStream = new FileInputStream(this.ksfile);
            }
            catch (FileNotFoundException ex) {
                if (this.command != Main.Command.GENKEYPAIR && this.command != Main.Command.GENSECKEY && this.command != Main.Command.IDENTITYDB && this.command != Main.Command.IMPORTCERT && this.command != Main.Command.IMPORTPASS && this.command != Main.Command.IMPORTKEYSTORE && this.command != Main.Command.PRINTCRL) {
                    throw new Exception(Main.rb.getString("Keystore.file.does.not.exist.") + this.ksfname);
                }
            }
        }
        if ((this.command == Main.Command.KEYCLONE || this.command == Main.Command.CHANGEALIAS) && this.dest == null) {
            this.dest = this.getAlias("destination");
            if ("".equals(this.dest)) {
                throw new Exception(Main.rb.getString("Must.specify.destination.alias"));
            }
        }
        if (this.command == Main.Command.DELETE && this.alias == null) {
            this.alias = this.getAlias(null);
            if ("".equals(this.alias)) {
                throw new Exception(Main.rb.getString("Must.specify.alias"));
            }
        }
        if (this.storetype == null) {
            this.storetype = KeyStore.getDefaultType();
        }
        if (this.providerName == null) {
            this.keyStore = KeyStore.getInstance(this.storetype);
        }
        else {
            this.keyStore = KeyStore.getInstance(this.storetype, this.providerName);
        }
        if (!this.nullStream) {
            if (this.inplaceImport) {
                this.keyStore.load(null, this.storePass);
            }
            else {
                this.keyStore.load(this.ksStream, this.storePass);
            }
            if (this.ksStream != null) {
                this.ksStream.close();
            }
        }
        if ("PKCS12".equalsIgnoreCase(this.storetype) && this.command == Main.Command.KEYPASSWD) {
            throw new UnsupportedOperationException(Main.rb.getString(".keypasswd.commands.not.supported.if.storetype.is.PKCS12"));
        }
        if (this.nullStream && this.storePass != null) {
            this.keyStore.load(null, this.storePass);
        }
        else if (!this.nullStream && this.storePass != null) {
            if (this.ksStream == null && this.storePass.length < 6) {
                throw new Exception(Main.rb.getString("Keystore.password.must.be.at.least.6.characters"));
            }
        }
        else if (this.storePass == null) {
            if (!this.protectedPath && !KeyStoreUtil.isWindowsKeyStore(this.storetype) && (this.command == Main.Command.CERTREQ || this.command == Main.Command.DELETE || this.command == Main.Command.GENKEYPAIR || this.command == Main.Command.GENSECKEY || this.command == Main.Command.IMPORTCERT || this.command == Main.Command.IMPORTPASS || this.command == Main.Command.IMPORTKEYSTORE || this.command == Main.Command.KEYCLONE || this.command == Main.Command.CHANGEALIAS || this.command == Main.Command.SELFCERT || this.command == Main.Command.STOREPASSWD || this.command == Main.Command.KEYPASSWD || this.command == Main.Command.IDENTITYDB)) {
                int n = 0;
                do {
                    if (this.command == Main.Command.IMPORTKEYSTORE) {
                        System.err.print(Main.rb.getString("Enter.destination.keystore.password."));
                    }
                    else {
                        System.err.print(Main.rb.getString("Enter.keystore.password."));
                    }
                    System.err.flush();
                    this.storePass = Password.readPassword(System.in);
                    this.passwords.add(this.storePass);
                    if (!this.nullStream && (this.storePass == null || this.storePass.length < 6)) {
                        System.err.println(Main.rb.getString("Keystore.password.is.too.short.must.be.at.least.6.characters"));
                        this.storePass = null;
                    }
                    if (this.storePass != null && !this.nullStream && this.ksStream == null) {
                        System.err.print(Main.rb.getString("Re.enter.new.password."));
                        char[] password = Password.readPassword(System.in);
                        this.passwords.add(password);
                        if (!Arrays.equals(this.storePass, password)) {
                            System.err.println(Main.rb.getString("They.don.t.match.Try.again"));
                            this.storePass = null;
                        }
                    }
                    ++n;
                } while (this.storePass == null && n < 3);
                if (this.storePass == null) {
                    System.err.println(Main.rb.getString("Too.many.failures.try.later"));
                    return;
                }
            }
            else if (!this.protectedPath && !KeyStoreUtil.isWindowsKeyStore(this.storetype) && this.isKeyStoreRelated(this.command) && this.command != Main.Command.PRINTCRL) {
                System.err.print(Main.rb.getString("Enter.keystore.password."));
                System.err.flush();
                this.storePass = Password.readPassword(System.in);
                this.passwords.add(this.storePass);
            }
            if (this.nullStream) {
                this.keyStore.load(null, this.storePass);
            }
            else if (this.ksStream != null) {
                this.ksStream = new FileInputStream(this.ksfile);
                this.keyStore.load(this.ksStream, this.storePass);
                this.ksStream.close();
            }
        }
        if (this.storePass != null && "PKCS12".equalsIgnoreCase(this.storetype)) {
            MessageFormat o2 = new MessageFormat(Main.rb.getString("Warning.Different.store.and.key.passwords.not.supported.for.PKCS12.KeyStores.Ignoring.user.specified.command.value."));
            if (this.keyPass != null && !Arrays.equals(this.storePass, this.keyPass)) {
                System.err.println(o2.format(new Object[] { "-keypass" }));
                this.keyPass = this.storePass;
            }
            if (this.newPass != null && !Arrays.equals(this.storePass, this.newPass)) {
                System.err.println(o2.format(new Object[] { "-new" }));
                this.newPass = this.storePass;
            }
            if (this.destKeyPass != null && !Arrays.equals(this.storePass, this.destKeyPass)) {
                System.err.println(o2.format(new Object[] { "-destkeypass" }));
                this.destKeyPass = this.storePass;
            }
        }
        if (this.command == Main.Command.PRINTCERT || this.command == Main.Command.IMPORTCERT || this.command == Main.Command.IDENTITYDB || this.command == Main.Command.PRINTCRL) {
            this.cf = CertificateFactory.getInstance("X509");
        }
        if (this.command != Main.Command.IMPORTCERT) {
            this.trustcacerts = false;
        }
        if (this.trustcacerts) {
            this.caks = KeyStoreUtil.getCacertsKeyStore();
        }
        if (this.command == Main.Command.CERTREQ) {
            if (this.filename != null) {
                try (final Object o7 = new PrintStream(new FileOutputStream(this.filename))) {
                    this.doCertReq(this.alias, this.sigAlgName, o7);
                }
            }
            else {
                this.doCertReq(this.alias, this.sigAlgName, printStream);
            }
            if (this.verbose && this.filename != null) {
                System.err.println(new MessageFormat(Main.rb.getString("Certification.request.stored.in.file.filename.")).format(new Object[] { this.filename }));
                System.err.println(Main.rb.getString("Submit.this.to.your.CA"));
            }
        }
        else if (this.command == Main.Command.DELETE) {
            this.doDeleteEntry(this.alias);
            this.kssave = true;
        }
        else if (this.command == Main.Command.EXPORTCERT) {
            if (this.filename != null) {
                try (final Object o6 = new PrintStream(new FileOutputStream(this.filename))) {
                    this.doExportCert(this.alias, o6);
                }
            }
            else {
                this.doExportCert(this.alias, printStream);
            }
            if (this.filename != null) {
                System.err.println(new MessageFormat(Main.rb.getString("Certificate.stored.in.file.filename.")).format(new Object[] { this.filename }));
            }
        }
        else if (this.command == Main.Command.GENKEYPAIR) {
            if (this.keyAlgName == null) {
                this.keyAlgName = "DSA";
            }
            this.doGenKeyPair(this.alias, this.dname, this.keyAlgName, this.keysize, this.sigAlgName);
            this.kssave = true;
        }
        else if (this.command == Main.Command.GENSECKEY) {
            if (this.keyAlgName == null) {
                this.keyAlgName = "DES";
            }
            this.doGenSecretKey(this.alias, this.keyAlgName, this.keysize);
            this.kssave = true;
        }
        else if (this.command == Main.Command.IMPORTPASS) {
            if (this.keyAlgName == null) {
                this.keyAlgName = "PBE";
            }
            this.doGenSecretKey(this.alias, this.keyAlgName, this.keysize);
            this.kssave = true;
        }
        else if (this.command == Main.Command.IDENTITYDB) {
            if (this.filename != null) {
                try (final Object o5 = new FileInputStream(this.filename)) {
                    this.doImportIdentityDatabase(o5);
                }
            }
            else {
                this.doImportIdentityDatabase(System.in);
            }
        }
        else if (this.command == Main.Command.IMPORTCERT) {
            InputStream in2 = System.in;
            if (this.filename != null) {
                in2 = new FileInputStream(this.filename);
            }
            String alias = (this.alias != null) ? this.alias : "mykey";
            try {
                if (this.keyStore.entryInstanceOf(alias, KeyStore.PrivateKeyEntry.class)) {
                    this.kssave = this.installReply(alias, in2);
                    if (this.kssave) {
                        System.err.println(Main.rb.getString("Certificate.reply.was.installed.in.keystore"));
                    }
                    else {
                        System.err.println(Main.rb.getString("Certificate.reply.was.not.installed.in.keystore"));
                    }
                }
                else if (!this.keyStore.containsAlias(alias) || this.keyStore.entryInstanceOf(alias, KeyStore.TrustedCertificateEntry.class)) {
                    this.kssave = this.addTrustedCert(alias, in2);
                    if (this.kssave) {
                        System.err.println(Main.rb.getString("Certificate.was.added.to.keystore"));
                    }
                    else {
                        System.err.println(Main.rb.getString("Certificate.was.not.added.to.keystore"));
                    }
                }
            }
            finally {
                if (in2 != System.in) {
                    in2.close();
                }
            }
        }
        else if (this.command == Main.Command.IMPORTKEYSTORE) {
            if (keyStore == null) {
                keyStore = this.loadSourceKeyStore();
            }
            this.doImportKeyStore(keyStore);
            this.kssave = true;
        }
        else if (this.command == Main.Command.KEYCLONE) {
            this.keyPassNew = this.newPass;
            if (this.alias == null) {
                this.alias = "mykey";
            }
            if (!this.keyStore.containsAlias(this.alias)) {
                throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { this.alias }));
            }
            if (!this.keyStore.entryInstanceOf(this.alias, KeyStore.PrivateKeyEntry.class)) {
                throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.references.an.entry.type.that.is.not.a.private.key.entry.The.keyclone.command.only.supports.cloning.of.private.key")).format(new Object[] { this.alias }));
            }
            this.doCloneEntry(this.alias, this.dest, true);
            this.kssave = true;
        }
        else if (this.command == Main.Command.CHANGEALIAS) {
            if (this.alias == null) {
                this.alias = "mykey";
            }
            this.doCloneEntry(this.alias, this.dest, false);
            if (this.keyStore.containsAlias(this.alias)) {
                this.doDeleteEntry(this.alias);
            }
            this.kssave = true;
        }
        else if (this.command == Main.Command.KEYPASSWD) {
            this.keyPassNew = this.newPass;
            this.doChangeKeyPasswd(this.alias);
            this.kssave = true;
        }
        else if (this.command == Main.Command.LIST) {
            if (this.storePass == null && !KeyStoreUtil.isWindowsKeyStore(this.storetype)) {
                this.printNoIntegrityWarning();
            }
            if (this.alias != null) {
                this.doPrintEntry(Main.rb.getString("the.certificate"), this.alias, printStream);
            }
            else {
                this.doPrintEntries(printStream);
            }
        }
        else if (this.command == Main.Command.PRINTCERT) {
            this.doPrintCert(printStream);
        }
        else if (this.command == Main.Command.SELFCERT) {
            this.doSelfCert(this.alias, this.dname, this.sigAlgName);
            this.kssave = true;
        }
        else if (this.command == Main.Command.STOREPASSWD) {
            this.storePassNew = this.newPass;
            if (this.storePassNew == null) {
                this.storePassNew = this.getNewPasswd("keystore password", this.storePass);
            }
            this.kssave = true;
        }
        else if (this.command == Main.Command.GENCERT) {
            if (this.alias == null) {
                this.alias = "mykey";
            }
            InputStream in = System.in;
            if (this.infilename != null) {
                in = new FileInputStream(this.infilename);
            }
            printStream = null;
            if (this.outfilename != null) {
                printStream = (printStream = new PrintStream(new FileOutputStream(this.outfilename)));
            }
            try {
                this.doGenCert(this.alias, this.sigAlgName, in, printStream);
            }
            finally {
                if (in != System.in) {
                    in.close();
                }
                if (printStream != null) {
                    printStream.close();
                }
            }
        }
        else if (this.command == Main.Command.GENCRL) {
            if (this.alias == null) {
                this.alias = "mykey";
            }
            if (this.filename != null) {
                try (final Object o4 = new PrintStream(new FileOutputStream(this.filename))) {
                    this.doGenCRL(o4);
                }
            }
            else {
                this.doGenCRL(printStream);
            }
        }
        else if (this.command == Main.Command.PRINTCERTREQ) {
            if (this.filename != null) {
                try (final Object o3 = new FileInputStream(this.filename)) {
                    this.doPrintCertReq(o3, printStream);
                }
            }
            else {
                this.doPrintCertReq(System.in, printStream);
            }
        }
        else if (this.command == Main.Command.PRINTCRL) {
            this.doPrintCRL(this.filename, printStream);
        }
        if (this.kssave) {
            if (this.verbose) {
                System.err.println(new MessageFormat(Main.rb.getString(".Storing.ksfname.")).format(new Object[] { this.nullStream ? "keystore" : this.ksfname }));
            }
            if (this.token) {
                this.keyStore.store(null, (char[])null);
            }
            else {
                char[] array = (this.storePassNew != null) ? this.storePassNew : this.storePass;
                if (this.nullStream) {
                    this.keyStore.store(null, array);
                }
                else {
                    ByteArrayOutputStream stream = new ByteArrayOutputStream();
                    this.keyStore.store(stream, array);
                    try (final Object o8 = new FileOutputStream(this.ksfname)) {
                        o8.write(stream.toByteArray());
                    }
                }
            }
        }
        if (this.isKeyStoreRelated(this.command) && !this.token && !this.nullStream && this.ksfname != null) {
            File o9 = new File(this.ksfname);
            if (o9.exists()) {
                String keyStoreType = this.keyStoreType(o9);
                if (keyStoreType.equalsIgnoreCase("JKS") || keyStoreType.equalsIgnoreCase("JCEKS")) {
                    boolean n2 = true;
                    Iterator<String> iterator2 = Collections.list(this.keyStore.aliases()).iterator();
                    while (iterator2.hasNext()) {
                        if (!this.keyStore.entryInstanceOf((String)iterator2.next(), KeyStore.TrustedCertificateEntry.class)) {
                            n2 = false;
                            break;
                        }
                        continue;
                    }
                    if (n2 == 0) {
                        this.weakWarnings.add(String.format(Main.rb.getString("jks.storetype.warning"), new Object[] { keyStoreType, this.ksfname }));
                    }
                }
                if (this.inplaceImport) {
                    String keyStoreType2 = this.keyStoreType(new File(this.inplaceBackupName));
                    this.weakWarnings.add(String.format(keyStoreType.equalsIgnoreCase(keyStoreType2) ? Main.rb.getString("backup.keystore.warning") : Main.rb.getString("migrate.keystore.warning"), new Object[] { this.srcksfname, keyStoreType2, this.inplaceBackupName, keyStoreType }));
                }
            }
        }
        return;
    }
    
    private String keyStoreType(File file) throws IOException {
        int n = -17957139;
        int n2 = -825307442;
        try (final DataInputStream dataInputStream = new DataInputStream(new FileInputStream(file))) {
            int int1 = dataInputStream.readInt();
            if (int1 == n) {
                return "JKS";
            }
            else if (int1 == n2) {
                return "JCEKS";
            }
            else {
                return "Non JKS/JCEKS";
            }
        }
    }
    
    private void doGenCert(String s, String compatibleSigAlgName, InputStream in, PrintStream printStream) throws Exception {
        if (!this.keyStore.containsAlias(s)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { s }));
        }
        Certificate certificate = this.keyStore.getCertificate(s);
        X500Name x500Name = (X500Name)((X509CertInfo)new X509CertImpl(certificate.getEncoded()).get("x509.info")).get("subject.dname");
        Date startDate = getStartDate(this.startDate);
        Date notAfter = new Date();
        notAfter.setTime(startDate.getTime() + this.validity * 1000L * 24L * 60L * 60L);
        CertificateValidity certificateValidity = new CertificateValidity(startDate, notAfter);
        PrivateKey key = (PrivateKey)this.recoverKey(s, this.storePass, this.keyPass).fst;
        if (compatibleSigAlgName == null) {
            compatibleSigAlgName = getCompatibleSigAlgName(key.getAlgorithm());
        }
        Signature instance = Signature.getInstance(compatibleSigAlgName);
        PSSParameterSpec defaultAlgorithmParameterSpec = AlgorithmId.getDefaultAlgorithmParameterSpec(compatibleSigAlgName, key);
        SignatureUtil.initSignWithParam(instance, key, defaultAlgorithmParameterSpec, null);
        X509CertInfo o = new X509CertInfo();
        AlgorithmId withParameterSpec = AlgorithmId.getWithParameterSpec(compatibleSigAlgName, defaultAlgorithmParameterSpec);
        o.set("validity", certificateValidity);
        o.set("serialNumber", new CertificateSerialNumber(new Random().nextInt() & Integer.MAX_VALUE));
        o.set("version", new CertificateVersion(2));
        o.set("algorithmID", new CertificateAlgorithmId(withParameterSpec));
        o.set("issuer", x500Name);
        BufferedReader o2 = new BufferedReader(new InputStreamReader(in));
        boolean n = false;
        StringBuffer buffer = new StringBuffer();
        while (true) {
            String line = o2.readLine();
            if (line == null) {
                break;
            }
            if (line.startsWith("-----BEGIN") && line.indexOf("REQUEST") >= 0) {
                n = true;
            }
            else {
                if (line.startsWith("-----END") && line.indexOf("REQUEST") >= 0) {
                    break;
                }
                if (!(n == 0)) {
                    buffer.append(line);
                }
                else {
                    continue;
                }
            }
        }
        PKCS10 o3 = new PKCS10(Pem.decode(new String(buffer)));
        this.checkWeak(Main.rb.getString("the.certificate.request"), o3);
        o.set("key", new CertificateX509Key(o3.getSubjectPublicKeyInfo()));
        o.set("subject", (this.dname == null) ? o3.getSubjectName() : new X500Name(this.dname));
        CertificateExtensions certificateExtensions = null;
        for (PKCS10Attribute pkcs10Attribute : o3.getAttributes().getAttributes()) {
            if (pkcs10Attribute.getAttributeId().equals(PKCS9Attribute.EXTENSION_REQUEST_OID)) {
                certificateExtensions = (CertificateExtensions)pkcs10Attribute.getAttributeValue();
            }
            else {
                continue;
            }
        }
        o.set("extensions", this.createV3Extensions(certificateExtensions, null, this.v3ext, o3.getSubjectPublicKeyInfo(), certificate.getPublicKey()));
        X509CertImpl o4 = new X509CertImpl(o);
        o4.sign(key, defaultAlgorithmParameterSpec, compatibleSigAlgName, null);
        this.dumpCert(o4, printStream);
        for (Certificate certificate2 : this.keyStore.getCertificateChain(s)) {
            if ((certificate2 instanceof X509Certificate)) {
                X509Certificate cert = (X509Certificate)certificate2;
                if (!KeyStoreUtil.isSelfSigned(cert)) {
                    this.dumpCert(cert, printStream);
                }
            }
        }
        this.checkWeak(Main.rb.getString("the.issuer"), this.keyStore.getCertificateChain(s));
        this.checkWeak(Main.rb.getString("the.generated.certificate"), o4);
        return;
    }
    
    private void doGenCRL(PrintStream printStream) throws Exception {
        if (this.ids == null) {
            throw new Exception("Must provide -id when -gencrl");
        }
        X500Name x500Name = (X500Name)((X509CertInfo)new X509CertImpl(this.keyStore.getCertificate(this.alias).getEncoded()).get("x509.info")).get("subject.dname");
        Date startDate = getStartDate(this.startDate);
        Date notAfter = (Date)startDate.clone();
        notAfter.setTime(notAfter.getTime() + this.validity * 1000L * 24L * 60L * 60L);
        CertificateValidity certificateValidity = new CertificateValidity(startDate, notAfter);
        PrivateKey privateKey = (PrivateKey)this.recoverKey(this.alias, this.storePass, this.keyPass).fst;
        if (this.sigAlgName == null) {
            this.sigAlgName = getCompatibleSigAlgName(privateKey.getAlgorithm());
        }
        X509CRLEntry[] array = new X509CRLEntry[this.ids.size()];
        for (int i = 0; i < this.ids.size(); ++i) {
            String s = (String)this.ids.get(i);
            int index = s.indexOf(58);
            if (index >= 0) {
                CRLExtensions crlEntryExts = new CRLExtensions();
                crlEntryExts.set("Reason", new CRLReasonCodeExtension(Integer.parseInt(s.substring(index + 1))));
                array[i] = new X509CRLEntryImpl(new BigInteger(s.substring(0, index)), startDate, crlEntryExts);
            }
            else {
                array[i] = new X509CRLEntryImpl(new BigInteger((String)this.ids.get(i)), startDate);
            }
        }
        X509CRLImpl o = new X509CRLImpl(x500Name, startDate, notAfter, array);
        o.sign(privateKey, this.sigAlgName);
        if (this.rfc) {
            printStream.println("-----BEGIN X509 CRL-----");
            printStream.println(Base64.getMimeEncoder(64, Main.CRLF).encodeToString(o.getEncodedInternal()));
            printStream.println("-----END X509 CRL-----");
        }
        else {
            printStream.write(o.getEncodedInternal());
        }
        this.checkWeak(Main.rb.getString("the.generated.crl"), o, privateKey);
        return;
    }
    
    private void doCertReq(String alias, String compatibleSigAlgName, PrintStream out) throws Exception {
        if (alias == null) {
            alias = "mykey";
        }
        Pair<Key, char[]> recoverKey = this.recoverKey(alias, this.storePass, this.keyPass);
        PrivateKey key = (PrivateKey)recoverKey.fst;
        if (this.keyPass == null) {
            this.keyPass = (char[])recoverKey.snd;
        }
        Certificate certificate = this.keyStore.getCertificate(alias);
        if (certificate == null) {
            throw new Exception(new MessageFormat(Main.rb.getString("alias.has.no.public.key.certificate.")).format(new Object[] { alias }));
        }
        PKCS10 o = new PKCS10(certificate.getPublicKey());
        o.getAttributes().setAttribute("extensions", new PKCS10Attribute(PKCS9Attribute.EXTENSION_REQUEST_OID, this.createV3Extensions(null, null, this.v3ext, certificate.getPublicKey(), null)));
        if (compatibleSigAlgName == null) {
            compatibleSigAlgName = getCompatibleSigAlgName(key.getAlgorithm());
        }
        Signature instance = Signature.getInstance(compatibleSigAlgName);
        SignatureUtil.initSignWithParam(instance, key, AlgorithmId.getDefaultAlgorithmParameterSpec(compatibleSigAlgName, key), null);
        o.encodeAndSign((this.dname == null) ? new X500Name(((X509Certificate)certificate).getSubjectDN().toString()) : new X500Name(this.dname), instance);
        o.print(out);
        this.checkWeak(Main.rb.getString("the.generated.certificate.request"), o);
        return;
    }
    
    private void doDeleteEntry(String s) throws Exception {
        if (!this.keyStore.containsAlias(s)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { s }));
        }
        this.keyStore.deleteEntry(s);
        return;
    }
    
    private void doExportCert(String s, PrintStream printStream) throws Exception {
        if (this.storePass == null && !KeyStoreUtil.isWindowsKeyStore(this.storetype)) {
            this.printNoIntegrityWarning();
        }
        if (s == null) {
            s = "mykey";
        }
        if (!this.keyStore.containsAlias(s)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { s }));
        }
        X509Certificate x509Certificate = (X509Certificate)this.keyStore.getCertificate(s);
        if (x509Certificate == null) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.has.no.certificate")).format(new Object[] { s }));
        }
        this.dumpCert(x509Certificate, printStream);
        this.checkWeak(Main.rb.getString("the.certificate"), x509Certificate);
        return;
    }
    
    private char[] promptForKeyPass(String s, String s2, char[] array) throws Exception {
        if ("PKCS12".equalsIgnoreCase(this.storetype)) {
            return array;
        }
        if (!this.token && !this.protectedPath) {
            int i = 0;
            while (i < 3) {
                System.err.println(new MessageFormat(Main.rb.getString("Enter.key.password.for.alias.")).format(new Object[] { s }));
                if (s2 == null) {
                    System.err.print(Main.rb.getString(".RETURN.if.same.as.keystore.password."));
                }
                else {
                    System.err.print(new MessageFormat(Main.rb.getString(".RETURN.if.same.as.for.otherAlias.")).format(new Object[] { s2 }));
                }
                System.err.flush();
                char[] password = Password.readPassword(System.in);
                this.passwords.add(password);
                if (password == null) {
                    return array;
                }
                if (password.length >= 6) {
                    System.err.print(Main.rb.getString("Re.enter.new.password."));
                    char[] password2 = Password.readPassword(System.in);
                    this.passwords.add(password2);
                    if (!Arrays.equals(password, password2)) {
                        System.err.println(Main.rb.getString("They.don.t.match.Try.again"));
                    }
                    else {
                        return password;
                    }
                }
                else {
                    System.err.println(Main.rb.getString("Key.password.is.too.short.must.be.at.least.6.characters"));
                }
                ++i;
            }
            if (i == 3) {
                if (this.command == Main.Command.KEYCLONE) {
                    throw new Exception(Main.rb.getString("Too.many.failures.Key.entry.not.cloned"));
                }
                throw new Exception(Main.rb.getString("Too.many.failures.key.not.added.to.keystore"));
            }
        }
        return null;
    }
    
    private char[] promptForCredential() throws Exception {
        if (System.console() == null) {
            char[] password3 = Password.readPassword(System.in);
            this.passwords.add(password3);
            return password3;
        }
        int i = 0;
        while (i < 3) {
            System.err.print(Main.rb.getString("Enter.the.password.to.be.stored."));
            System.err.flush();
            char[] password = Password.readPassword(System.in);
            this.passwords.add(password);
            System.err.print(Main.rb.getString("Re.enter.password."));
            char[] password2 = Password.readPassword(System.in);
            this.passwords.add(password2);
            if (!Arrays.equals(password, password2)) {
                System.err.println(Main.rb.getString("They.don.t.match.Try.again"));
                ++i;
            }
            else {
                return password;
            }
        }
        if (i == 3) {
            throw new Exception(Main.rb.getString("Too.many.failures.key.not.added.to.keystore"));
        }
        return null;
    }
    
    private void doGenSecretKey(String alias, String protectionAlgorithm, int n) throws Exception {
        if (alias == null) {
            alias = "mykey";
        }
        if (this.keyStore.containsAlias(alias)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Secret.key.not.generated.alias.alias.already.exists")).format(new Object[] { alias }));
        }
        boolean n2 = true;
        SecretKey secretKey;
        if (protectionAlgorithm.toUpperCase(Locale.ENGLISH).startsWith("PBE")) {
            secretKey = SecretKeyFactory.getInstance("PBE").generateSecret(new PBEKeySpec(this.promptForCredential()));
            if (!"PBE".equalsIgnoreCase(protectionAlgorithm)) {
                n2 = false;
            }
            if (this.verbose) {
                System.err.println(new MessageFormat(Main.rb.getString("Generated.keyAlgName.secret.key")).format(new Object[] { n2 ? "PBE" : secretKey.getAlgorithm() }));
            }
        }
        else {
            KeyGenerator instance = KeyGenerator.getInstance(protectionAlgorithm);
            if (n == -1) {
                if ("DES".equalsIgnoreCase(protectionAlgorithm)) {
                    n = 56;
                }
                else if ("DESede".equalsIgnoreCase(protectionAlgorithm)) {
                    n = 168;
                }
                else {
                    throw new Exception(Main.rb.getString("Please.provide.keysize.for.secret.key.generation"));
                }
            }
            instance.init(n);
            secretKey = instance.generateKey();
            if (this.verbose) {
                System.err.println(new MessageFormat(Main.rb.getString("Generated.keysize.bit.keyAlgName.secret.key")).format(new Object[] { new Integer(n), secretKey.getAlgorithm() }));
            }
        }
        if (this.keyPass == null) {
            this.keyPass = this.promptForKeyPass(alias, null, this.storePass);
        }
        if (!(n2 == 0)) {
            this.keyStore.setKeyEntry(alias, secretKey, this.keyPass, (Certificate[])null);
        }
        else {
            this.keyStore.setEntry(alias, new KeyStore.SecretKeyEntry(secretKey), new KeyStore.PasswordProtection(this.keyPass, protectionAlgorithm, null));
        }
        return;
    }
    
    private static String getCompatibleSigAlgName(String anotherString) throws Exception {
        if ("DSA".equalsIgnoreCase(anotherString)) {
            return "SHA256WithDSA";
        }
        if ("RSA".equalsIgnoreCase(anotherString)) {
            return "SHA256WithRSA";
        }
        if ("EC".equalsIgnoreCase(anotherString)) {
            return "SHA256withECDSA";
        }
        throw new Exception(Main.rb.getString("Cannot.derive.signature.algorithm"));
    }
    
    private void doGenKeyPair(String s, String dname, String s2, int value, String compatibleSigAlgName) throws Exception {
        if (value == -1) {
            if ("EC".equalsIgnoreCase(s2)) {
                value = SecurityProviderConstants.DEF_EC_KEY_SIZE;
            }
            else if ("RSA".equalsIgnoreCase(s2)) {
                value = SecurityProviderConstants.DEF_RSA_KEY_SIZE;
            }
            else if ("RSASSA-PSS".equalsIgnoreCase(s2)) {
                value = SecurityProviderConstants.DEF_RSASSA_PSS_KEY_SIZE;
            }
            else if ("DSA".equalsIgnoreCase(s2)) {
                value = SecurityProviderConstants.DEF_DSA_KEY_SIZE;
            }
        }
        if (s == null) {
            s = "mykey";
        }
        if (this.keyStore.containsAlias(s)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Key.pair.not.generated.alias.alias.already.exists")).format(new Object[] { s }));
        }
        if (compatibleSigAlgName == null) {
            compatibleSigAlgName = getCompatibleSigAlgName(s2);
        }
        CertAndKeyGen o = new CertAndKeyGen(s2, compatibleSigAlgName, this.providerName);
        X500Name x500Name;
        if (dname == null) {
            x500Name = this.getX500Name();
        }
        else {
            x500Name = new X500Name(dname);
        }
        o.generate(value);
        PrivateKey privateKey = o.getPrivateKey();
        X509Certificate[] chain = { o.getSelfCertificate(x500Name, getStartDate(this.startDate), this.validity * 24L * 60L * 60L, this.createV3Extensions(null, null, this.v3ext, o.getPublicKeyAnyway(), null)) };
        if (this.verbose) {
            System.err.println(new MessageFormat(Main.rb.getString("Generating.keysize.bit.keyAlgName.key.pair.and.self.signed.certificate.sigAlgName.with.a.validity.of.validality.days.for")).format(new Object[] { new Integer(value), privateKey.getAlgorithm(), chain[0].getSigAlgName(), new Long(this.validity), x500Name }));
        }
        if (this.keyPass == null) {
            this.keyPass = this.promptForKeyPass(s, null, this.storePass);
        }
        this.checkWeak(Main.rb.getString("the.generated.certificate"), chain[0]);
        this.keyStore.setKeyEntry(s, privateKey, this.keyPass, chain);
        return;
    }
    
    private void doCloneEntry(String s, String s2, boolean b) throws Exception {
        if (s == null) {
            s = "mykey";
        }
        if (this.keyStore.containsAlias(s2)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Destination.alias.dest.already.exists")).format(new Object[] { s2 }));
        }
        Pair<KeyStore.Entry, char[]> recoverEntry = this.recoverEntry(this.keyStore, s, this.storePass, this.keyPass);
        KeyStore.Entry entry = (KeyStore.Entry)recoverEntry.fst;
        this.keyPass = (char[])recoverEntry.snd;
        Object protParam = null;
        if (this.keyPass != null) {
            if (!b || "PKCS12".equalsIgnoreCase(this.storetype)) {
                this.keyPassNew = this.keyPass;
            }
            else if (this.keyPassNew == null) {
                this.keyPassNew = this.promptForKeyPass(s2, s, this.keyPass);
            }
            protParam = new KeyStore.PasswordProtection(this.keyPassNew);
        }
        this.keyStore.setEntry(s2, entry, protParam);
        return;
    }
    
    private void doChangeKeyPasswd(String s) throws Exception {
        if (s == null) {
            s = "mykey";
        }
        Pair<Key, char[]> recoverKey = this.recoverKey(s, this.storePass, this.keyPass);
        Key key = (Key)recoverKey.fst;
        if (this.keyPass == null) {
            this.keyPass = (char[])recoverKey.snd;
        }
        if (this.keyPassNew == null) {
            this.keyPassNew = this.getNewPasswd(new MessageFormat(Main.rb.getString("key.password.for.alias.")).format(new Object[] { s }), this.keyPass);
        }
        this.keyStore.setKeyEntry(s, key, this.keyPassNew, this.keyStore.getCertificateChain(s));
    }
    
    private void doImportIdentityDatabase(InputStream inputStream) throws Exception {
        System.err.println(Main.rb.getString("No.entries.from.identity.database.added"));
    }
    
    private void doPrintEntry(String s, String s2, PrintStream printStream) throws Exception {
        if (!this.keyStore.containsAlias(s2)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { s2 }));
        }
        if (this.verbose || this.rfc || this.debug) {
            printStream.println(new MessageFormat(Main.rb.getString("Alias.name.alias")).format(new Object[] { s2 }));
            if (!this.token) {
                printStream.println(new MessageFormat(Main.rb.getString("Creation.date.keyStore.getCreationDate.alias.")).format(new Object[] { this.keyStore.getCreationDate(s2) }));
            }
        }
        else if (!this.token) {
            printStream.print(new MessageFormat(Main.rb.getString("alias.keyStore.getCreationDate.alias.")).format(new Object[] { s2, this.keyStore.getCreationDate(s2) }));
        }
        else {
            printStream.print(new MessageFormat(Main.rb.getString("alias.")).format(new Object[] { s2 }));
        }
        if (this.keyStore.entryInstanceOf(s2, KeyStore.SecretKeyEntry.class)) {
            if (this.verbose || this.rfc || this.debug) {
                printStream.println(new MessageFormat(Main.rb.getString("Entry.type.type.")).format(new Object[] { "SecretKeyEntry" }));
            }
            else {
                printStream.println("SecretKeyEntry, ");
            }
        }
        else if (this.keyStore.entryInstanceOf(s2, KeyStore.PrivateKeyEntry.class)) {
            if (this.verbose || this.rfc || this.debug) {
                printStream.println(new MessageFormat(Main.rb.getString("Entry.type.type.")).format(new Object[] { "PrivateKeyEntry" }));
            }
            else {
                printStream.println("PrivateKeyEntry, ");
            }
            Certificate[] certificateChain = this.keyStore.getCertificateChain(s2);
            if (certificateChain != null) {
                if (this.verbose || this.rfc || this.debug) {
                    printStream.println(Main.rb.getString("Certificate.chain.length.") + certificateChain.length);
                    for (int i = 0; i < certificateChain.length; ++i) {
                        printStream.println(new MessageFormat(Main.rb.getString("Certificate.i.1.")).format(new Object[] { new Integer(i + 1) }));
                        if (this.verbose && certificateChain[i] instanceof X509Certificate) {
                            this.printX509Cert((X509Certificate)(X509Certificate)certificateChain[i], printStream);
                        }
                        else if (this.debug) {
                            printStream.println(certificateChain[i].toString());
                        }
                        else {
                            this.dumpCert(certificateChain[i], printStream);
                        }
                        this.checkWeak(s, certificateChain[i]);
                    }
                }
                else {
                    printStream.println(Main.rb.getString("Certificate.fingerprint.SHA.256.") + this.getCertFingerPrint("SHA-256", certificateChain[0]));
                    this.checkWeak(s, certificateChain[0]);
                }
            }
        }
        else if (this.keyStore.entryInstanceOf(s2, KeyStore.TrustedCertificateEntry.class)) {
            Certificate certificate = this.keyStore.getCertificate(s2);
            String string = new MessageFormat(Main.rb.getString("Entry.type.type.")).format(new Object[] { "trustedCertEntry" }) + "\n";
            if (this.verbose && certificate instanceof X509Certificate) {
                printStream.println(string);
                this.printX509Cert((X509Certificate)certificate, printStream);
            }
            else if (this.rfc) {
                printStream.println(string);
                this.dumpCert(certificate, printStream);
            }
            else if (this.debug) {
                printStream.println(certificate.toString());
            }
            else {
                printStream.println("trustedCertEntry, ");
                printStream.println(Main.rb.getString("Certificate.fingerprint.SHA.256.") + this.getCertFingerPrint("SHA-256", certificate));
            }
            this.checkWeak(s, certificate);
        }
        else {
            printStream.println(Main.rb.getString("Unknown.Entry.Type"));
        }
        return;
    }
    
    boolean inplaceImportCheck() throws Exception {
        if ("PKCS11".equalsIgnoreCase(this.srcstoretype) || KeyStoreUtil.isWindowsKeyStore(this.srcstoretype)) {
            return false;
        }
        if (this.srcksfname != null) {
            File file = new File(this.srcksfname);
            if (file.exists() && file.length() == 0L) {
                throw new Exception(Main.rb.getString("Source.keystore.file.exists.but.is.empty.") + this.srcksfname);
            }
            if (file.getCanonicalFile().equals(new File(this.ksfname).getCanonicalFile())) {
                return true;
            }
            System.err.println(String.format(Main.rb.getString("importing.keystore.status"), new Object[] { this.srcksfname, this.ksfname }));
            return false;
        }
        else {
            throw new Exception(Main.rb.getString("Please.specify.srckeystore"));
        }
    }
    
    KeyStore loadSourceKeyStore() throws Exception {
        FileInputStream stream = null;
        if ("PKCS11".equalsIgnoreCase(this.srcstoretype) || KeyStoreUtil.isWindowsKeyStore(this.srcstoretype)) {
            if (!"NONE".equals(this.srcksfname)) {
                System.err.println(MessageFormat.format(Main.rb.getString(".keystore.must.be.NONE.if.storetype.is.{0}"), new Object[] { this.srcstoretype }));
                System.err.println();
                this.tinyHelp();
            }
        }
        else {
            stream = new FileInputStream(new File(this.srcksfname));
        }
        KeyStore keyStore;
        try {
            if (this.srcstoretype == null) {
                this.srcstoretype = KeyStore.getDefaultType();
            }
            if (this.srcProviderName == null) {
                keyStore = KeyStore.getInstance(this.srcstoretype);
            }
            else {
                keyStore = KeyStore.getInstance(this.srcstoretype, this.srcProviderName);
            }
            if (this.srcstorePass == null && !this.srcprotectedPath && !KeyStoreUtil.isWindowsKeyStore(this.srcstoretype)) {
                System.err.print(Main.rb.getString("Enter.source.keystore.password."));
                System.err.flush();
                this.srcstorePass = Password.readPassword(System.in);
                this.passwords.add(this.srcstorePass);
            }
            if ("PKCS12".equalsIgnoreCase(this.srcstoretype) && this.srckeyPass != null && this.srcstorePass != null && !Arrays.equals(this.srcstorePass, this.srckeyPass)) {
                System.err.println(new MessageFormat(Main.rb.getString("Warning.Different.store.and.key.passwords.not.supported.for.PKCS12.KeyStores.Ignoring.user.specified.command.value.")).format(new Object[] { "-srckeypass" }));
                this.srckeyPass = this.srcstorePass;
            }
            keyStore.load(stream, this.srcstorePass);
        }
        finally {
            if (stream != null) {
                stream.close();
            }
        }
        if (this.srcstorePass == null && !KeyStoreUtil.isWindowsKeyStore(this.srcstoretype)) {
            System.err.println();
            System.err.println(Main.rb.getString(".WARNING.WARNING.WARNING."));
            System.err.println(Main.rb.getString(".The.integrity.of.the.information.stored.in.the.srckeystore."));
            System.err.println(Main.rb.getString(".WARNING.WARNING.WARNING."));
            System.err.println();
            return keyStore;
        }
        return keyStore;
    }
    
    private void doImportKeyStore(KeyStore keyStore) throws Exception {
        if (this.alias != null) {
            this.doImportKeyStoreSingle(keyStore, this.alias);
        }
        else {
            if (this.dest != null || this.srckeyPass != null) {
                throw new Exception(Main.rb.getString("if.alias.not.specified.destalias.and.srckeypass.must.not.be.specified"));
            }
            this.doImportKeyStoreAll(keyStore);
        }
        if (this.inplaceImport) {
            int i = 1;
            Object o;
            while (true) {
                this.inplaceBackupName = this.srcksfname + ".old" + ((i == 1) ? "" : Integer.valueOf(i));
                o = new File(this.inplaceBackupName);
                if (!o.exists()) {
                    break;
                }
                ++i;
            }
            Files.copy(Paths.get(this.srcksfname, new String[0]), o.toPath(), new CopyOption[0]);
        }
    }
    
    private int doImportKeyStoreSingle(KeyStore keyStore, String alias) throws Exception {
        String inputStringFromStdin = (this.dest == null) ? alias : this.dest;
        if (this.keyStore.containsAlias(inputStringFromStdin)) {
            Object[] obj = { alias };
            if (this.noprompt) {
                System.err.println(new MessageFormat(Main.rb.getString("Warning.Overwriting.existing.alias.alias.in.destination.keystore")).format(obj));
            }
            else if ("NO".equals(this.getYesNoReply(new MessageFormat(Main.rb.getString("Existing.entry.alias.alias.exists.overwrite.no.")).format(obj)))) {
                inputStringFromStdin = this.inputStringFromStdin(Main.rb.getString("Enter.new.alias.name.RETURN.to.cancel.import.for.this.entry."));
                if ("".equals(inputStringFromStdin)) {
                    System.err.println(new MessageFormat(Main.rb.getString("Entry.for.alias.alias.not.imported.")).format(obj));
                    return 0;
                }
            }
        }
        Pair<KeyStore.Entry, char[]> recoverEntry = this.recoverEntry(keyStore, alias, this.srcstorePass, this.srckeyPass);
        KeyStore.Entry entry = (KeyStore.Entry)recoverEntry.fst;
        Object protParam = null;
        char[] destKeyPass = null;
        if (this.destKeyPass != null) {
            destKeyPass = this.destKeyPass;
            protParam = new KeyStore.PasswordProtection(this.destKeyPass);
        }
        else if (recoverEntry.snd != null) {
            destKeyPass = (char[])recoverEntry.snd;
            protParam = new KeyStore.PasswordProtection((char[])recoverEntry.snd);
        }
        try {
            Certificate certificate = keyStore.getCertificate(alias);
            if (certificate != null) {
                this.checkWeak("<" + inputStringFromStdin + ">", certificate);
            }
            this.keyStore.setEntry(inputStringFromStdin, entry, protParam);
            if ("PKCS12".equalsIgnoreCase(this.storetype) && destKeyPass != null && !Arrays.equals(destKeyPass, this.storePass)) {
                throw new Exception(Main.rb.getString("The.destination.pkcs12.keystore.has.different.storepass.and.keypass.Please.retry.with.destkeypass.specified."));
            }
            return 1;
        }
        catch (KeyStoreException ex) {
            System.err.println(new MessageFormat(Main.rb.getString("Problem.importing.entry.for.alias.alias.exception.Entry.for.alias.alias.not.imported.")).format(new Object[] { alias, ex.toString() }));
            return 2;
        }
    }
    
    private void doImportKeyStoreAll(KeyStore keyStore) throws Exception {
        int i = 0;
        int size = keyStore.size();
        Enumeration<String> aliases = keyStore.aliases();
        while (aliases.hasMoreElements()) {
            String s = (String)aliases.nextElement();
            int doImportKeyStoreSingle = this.doImportKeyStoreSingle(keyStore, s);
            if (doImportKeyStoreSingle == 1) {
                ++i;
                System.err.println(new MessageFormat(Main.rb.getString("Entry.for.alias.alias.successfully.imported.")).format(new Object[] { s }));
            }
            else {
                if (doImportKeyStoreSingle == 2 && !this.noprompt && "YES".equals(this.getYesNoReply("Do you want to quit the import process? [no]:  "))) {
                    break;
                }
                continue;
            }
        }
        System.err.println(new MessageFormat(Main.rb.getString("Import.command.completed.ok.entries.successfully.imported.fail.entries.failed.or.cancelled")).format(new Object[] { Integer.valueOf(i), Integer.valueOf(size - i) }));
    }
    
    private void doPrintEntries(PrintStream printStream) throws Exception {
        String type = this.keyStore.getType();
        if ("JKS".equalsIgnoreCase(type) && this.ksfile != null && this.ksfile.exists() && !"JKS".equalsIgnoreCase(this.keyStoreType(this.ksfile))) {
            type = "PKCS12";
        }
        printStream.println(Main.rb.getString("Keystore.type.") + type);
        printStream.println(Main.rb.getString("Keystore.provider.") + this.keyStore.getProvider().getName());
        printStream.println();
        printStream.println(((this.keyStore.size() == 1) ? new MessageFormat(Main.rb.getString("Your.keystore.contains.keyStore.size.entry")) : new MessageFormat(Main.rb.getString("Your.keystore.contains.keyStore.size.entries"))).format(new Object[] { new Integer(this.keyStore.size()) }));
        printStream.println();
        Enumeration<String> aliases = this.keyStore.aliases();
        while (aliases.hasMoreElements()) {
            String str = (String)aliases.nextElement();
            this.doPrintEntry("<" + str + ">", str, printStream);
            if (this.verbose || this.rfc) {
                printStream.println(Main.rb.getString("NEWLINE"));
                printStream.println(Main.rb.getString("STAR"));
                printStream.println(Main.rb.getString("STARNN"));
            }
            else {
                continue;
            }
        }
    }
    
    private static <T> Iterable<T> e2i(final Enumeration<T> enumeration) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {
                    @Override
                    public boolean hasNext() {
                        return enumeration.hasMoreElements();
                    }
                    
                    @Override
                    public T next() {
                        return enumeration.nextElement();
                    }
                    
                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException("Not supported yet.");
                    }
                };
            }
        };
    }
    
    public static Collection<? extends CRL> loadCRLs(String s) throws Exception {
        InputStream inputStream = null;
        URI uri = null;
        if (s == null) {
            inputStream = System.in;
        }
        else {
            try {
                uri = new URI(s);
                if (!uri.getScheme().equals("ldap")) {
                    inputStream = uri.toURL().openStream();
                }
            }
            catch (Exception ex) {
                try {
                    inputStream = new FileInputStream(s);
                }
                catch (Exception ex2) {
                    if (uri == null || uri.getScheme() == null) {
                        throw ex2;
                    }
                    throw ex;
                }
            }
        }
        if (inputStream != null) {
            try {
                ByteArrayOutputStream o = new ByteArrayOutputStream();
                byte[] array = new byte[4096];
                while (true) {
                    int read = inputStream.read(array);
                    if (read < 0) {
                        break;
                    }
                    o.write(array, 0, read);
                }
                Collection<Object> generateCRLs = CertificateFactory.getInstance("X509").generateCRLs(new ByteArrayInputStream(o.toByteArray()));
            }
            finally {
                if (inputStream != System.in) {
                    inputStream.close();
                }
            }
        }
        CertStoreHelper instance = CertStoreHelper.getInstance("LDAP");
        String s2 = uri.getPath();
        if (s2.charAt(0) == '/') {
            s2 = s2.substring(1);
        }
        return instance.getCertStore(uri).getCRLs(instance.wrap(new X509CRLSelector(), null, s2));
    }
    
    public static List<CRL> readCRLsFromCert(X509Certificate cert) throws Exception {
        ArrayList<X509CRL> list = new ArrayList();
        CRLDistributionPointsExtension crlDistributionPointsExtension = X509CertImpl.toImpl(cert).getCRLDistributionPointsExtension();
        if (crlDistributionPointsExtension == null) {
            return list;
        }
        Iterator<Object> iterator = crlDistributionPointsExtension.get("points").iterator();
        while (iterator.hasNext()) {
            GeneralNames fullName = ((DistributionPoint)iterator.next()).getFullName();
            if (fullName != null) {
                for (GeneralName generalName : fullName.names()) {
                    if (generalName.getType() == 6) {
                        for (CRL crl : loadCRLs(((URIName)generalName.getName()).getName())) {
                            if ((crl instanceof X509CRL)) {
                                list.add((X509CRL)crl);
                            }
                            else {
                                continue;
                            }
                        }
                        break;
                    }
                    continue;
                }
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    private static String verifyCRL(KeyStore keyStore, CRL crl) throws Exception {
        X500Principal issuerX500Principal = ((X509CRLImpl)crl).getIssuerX500Principal();
        for (String alias : e2i(keyStore.aliases())) {
            Certificate certificate = keyStore.getCertificate(alias);
            if (certificate instanceof X509Certificate && ((X509Certificate)certificate).getSubjectX500Principal().equals(issuerX500Principal)) {
                try {
                    ((X509CRLImpl)crl).verify(certificate.getPublicKey());
                    return alias;
                }
                catch (Exception ex) {}
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    private void doPrintCRL(String s, PrintStream printStream) throws Exception {
        for (CRL crl : loadCRLs(s)) {
            this.printCRL(crl, printStream);
            String s2 = null;
            Certificate certificate = null;
            if (this.caks != null) {
                s2 = verifyCRL(this.caks, crl);
                if (s2 != null) {
                    certificate = this.caks.getCertificate(s2);
                    printStream.printf(Main.rb.getString("verified.by.s.in.s.weak"), new Object[] { s2, "cacerts", this.withWeak(certificate.getPublicKey()) });
                    printStream.println();
                }
            }
            if (s2 == null && this.keyStore != null) {
                s2 = verifyCRL(this.keyStore, crl);
                if (s2 != null) {
                    certificate = this.keyStore.getCertificate(s2);
                    printStream.printf(Main.rb.getString("verified.by.s.in.s.weak"), new Object[] { s2, "keystore", this.withWeak(certificate.getPublicKey()) });
                    printStream.println();
                }
            }
            if (s2 == null) {
                printStream.println(Main.rb.getString("STAR"));
                printStream.println(Main.rb.getString("warning.not.verified.make.sure.keystore.is.correct"));
                printStream.println(Main.rb.getString("STARNN"));
            }
            this.checkWeak(Main.rb.getString("the.crl"), crl, (certificate == null) ? null : certificate.getPublicKey());
        }
    }
    
    private void printCRL(CRL crl, PrintStream printStream) throws Exception {
        X509CRL x509CRL = (X509CRL)crl;
        if (this.rfc) {
            printStream.println("-----BEGIN X509 CRL-----");
            printStream.println(Base64.getMimeEncoder(64, Main.CRLF).encodeToString(x509CRL.getEncoded()));
            printStream.println("-----END X509 CRL-----");
        }
        else {
            String x;
            if ((crl instanceof X509CRLImpl)) {
                X509CRLImpl x509CRLImpl = (X509CRLImpl)crl;
                x = x509CRLImpl.toStringWithAlgName(this.withWeak("" + x509CRLImpl.getSigAlgId()));
            }
            else {
                x = crl.toString();
            }
            printStream.println(x);
        }
    }
    
    private void doPrintCertReq(InputStream in, PrintStream printStream) throws Exception {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in));
        StringBuffer buffer = new StringBuffer();
        int n = 0;
        while (true) {
            String line = bufferedReader.readLine();
            if (line == null) {
                break;
            }
            if (n == 0) {
                if (line.startsWith("-----")) {
                    n = 1;
                }
                else {
                    continue;
                }
            }
            else {
                if (line.startsWith("-----")) {
                    break;
                }
                buffer.append(line);
            }
        }
        PKCS10 x = new PKCS10(Pem.decode(new String(buffer)));
        PublicKey subjectPublicKeyInfo = x.getSubjectPublicKeyInfo();
        printStream.printf(Main.rb.getString("PKCS.10.with.weak"), new Object[] { x.getSubjectName(), subjectPublicKeyInfo.getFormat(), this.withWeak(subjectPublicKeyInfo), this.withWeak(x.getSigAlg()) });
        for (PKCS10Attribute pkcs10Attribute : x.getAttributes().getAttributes()) {
            if (pkcs10Attribute.getAttributeId().equals(PKCS9Attribute.EXTENSION_REQUEST_OID)) {
                CertificateExtensions certificateExtensions = (CertificateExtensions)pkcs10Attribute.getAttributeValue();
                if (certificateExtensions != null) {
                    printExtensions(Main.rb.getString("Extension.Request."), certificateExtensions, printStream);
                }
                else {
                    continue;
                }
            }
            else {
                printStream.println("Attribute: " + pkcs10Attribute.getAttributeId());
                printStream.print(new PKCS9Attribute(pkcs10Attribute.getAttributeId(), pkcs10Attribute.getAttributeValue()).getName() + ": ");
                Object attributeValue = pkcs10Attribute.getAttributeValue();
                printStream.println((attributeValue instanceof String[]) ? Arrays.toString((Object[])(Object[])attributeValue) : attributeValue);
            }
        }
        if (this.debug) {
            printStream.println(x);
        }
        this.checkWeak(Main.rb.getString("the.certificate.request"), x);
    }
    
    private void printCertFromStream(InputStream inStream, PrintStream printStream) throws Exception {
        Collection<Object> generateCertificates;
        try {
            generateCertificates = this.cf.generateCertificates(inStream);
        }
        catch (CertificateException cause) {
            throw new Exception(Main.rb.getString("Failed.to.parse.input"), cause);
        }
        if (generateCertificates.isEmpty()) {
            throw new Exception(Main.rb.getString("Empty.input"));
        }
        Certificate[] array = (Certificate[])generateCertificates.toArray(new Certificate[generateCertificates.size()]);
        for (int i = 0; i < array.length; ++i) {
            X509Certificate x509Certificate;
            try {
                x509Certificate = (X509Certificate)array[i];
            }
            catch (ClassCastException ex) {
                throw new Exception(Main.rb.getString("Not.X.509.certificate"));
            }
            if (array.length > 1) {
                printStream.println(new MessageFormat(Main.rb.getString("Certificate.i.1.")).format(new Object[] { new Integer(i + 1) }));
            }
            if (this.rfc) {
                this.dumpCert(x509Certificate, printStream);
            }
            else {
                this.printX509Cert(x509Certificate, printStream);
            }
            if (i < array.length - 1) {
                printStream.println();
            }
            this.checkWeak(oneInMany(Main.rb.getString("the.certificate"), i, array.length), x509Certificate);
        }
        return;
    }
    
    private static String oneInMany(String s, int n, int i) {
        if (i == 1) {
            return s;
        }
        return String.format(Main.rb.getString("one.in.many"), new Object[] { s, Integer.valueOf(n + 1), Integer.valueOf(i) });
    }
    
    private void doPrintCert(PrintStream printStream) throws Exception {
        if (this.jarfile != null) {
            JarFile jarFile = new JarFile(this.jarfile, true);
            Enumeration<JarEntry> entries = jarFile.entries();
            HashSet<CodeSigner> set = new HashSet();
            byte[] b = new byte[8192];
            int n = 0;
            while (entries.hasMoreElements()) {
                JarEntry ze = (JarEntry)entries.nextElement();
                try (final InputStream inputStream = jarFile.getInputStream(ze)) {
                    while (inputStream.read(b) != -1) {}
                }
                CodeSigner[] codeSigners = ze.getCodeSigners();
                if (codeSigners != null) {
                    for (CodeSigner codeSigner : codeSigners) {
                        if (!set.contains(codeSigner)) {
                            set.add(codeSigner);
                            printStream.printf(Main.rb.getString("Signer.d."), new Object[] { Integer.valueOf(++n) });
                            printStream.println();
                            printStream.println();
                            printStream.println(Main.rb.getString("Signature."));
                            printStream.println();
                            List<Object> certificates2 = codeSigner.getSignerCertPath().getCertificates();
                            int n2 = 0;
                            Iterator<Object> iterator2 = certificates2.iterator();
                            while (iterator2.hasNext()) {
                                X509Certificate x509Certificate = (X509Certificate)(Certificate)iterator2.next();
                                if (this.rfc) {
                                    printStream.println(Main.rb.getString("Certificate.owner.") + x509Certificate.getSubjectDN() + "\n");
                                    this.dumpCert(x509Certificate, printStream);
                                }
                                else {
                                    this.printX509Cert(x509Certificate, printStream);
                                }
                                printStream.println();
                                this.checkWeak(oneInMany(Main.rb.getString("the.certificate"), n2++, certificates2.size()), x509Certificate);
                            }
                            Timestamp timestamp = codeSigner.getTimestamp();
                            if (timestamp != null) {
                                printStream.println(Main.rb.getString("Timestamp."));
                                printStream.println();
                                List<Object> certificates3 = timestamp.getSignerCertPath().getCertificates();
                                int n3 = 0;
                                Iterator<Object> iterator3 = certificates3.iterator();
                                while (iterator3.hasNext()) {
                                    X509Certificate x509Certificate2 = (X509Certificate)(Certificate)iterator3.next();
                                    if (this.rfc) {
                                        printStream.println(Main.rb.getString("Certificate.owner.") + x509Certificate2.getSubjectDN() + "\n");
                                        this.dumpCert(x509Certificate2, printStream);
                                    }
                                    else {
                                        this.printX509Cert(x509Certificate2, printStream);
                                    }
                                    printStream.println();
                                    this.checkWeak(oneInMany(Main.rb.getString("the.tsa.certificate"), n3++, certificates3.size()), x509Certificate2);
                                }
                            }
                        }
                    }
                }
                else {
                    continue;
                }
            }
            jarFile.close();
            if (set.isEmpty()) {
                printStream.println(Main.rb.getString("Not.a.signed.jar.file"));
            }
        }
        else if (this.sslserver != null) {
            CertStore certStore = CertStoreHelper.getInstance("SSLServer").getCertStore(new URI("https://" + this.sslserver));
            Collection<Object> certificates;
            try {
                certificates = certStore.getCertificates(null);
                if (certificates.isEmpty()) {
                    throw new Exception(Main.rb.getString("No.certificate.from.the.SSL.server"));
                }
            }
            catch (CertStoreException ex) {
                if ((ex.getCause() instanceof IOException)) {
                    throw new Exception(Main.rb.getString("No.certificate.from.the.SSL.server"), ex.getCause());
                }
                throw ex;
            }
            int i = 0;
            for (Certificate certificate : certificates) {
                try {
                    if (this.rfc) {
                        this.dumpCert(certificate, printStream);
                    }
                    else {
                        printStream.println("Certificate #" + i);
                        printStream.println("====================================");
                        this.printX509Cert((X509Certificate)certificate, printStream);
                        printStream.println();
                    }
                    this.checkWeak(oneInMany(Main.rb.getString("the.certificate"), i++, certificates.size()), certificate);
                }
                catch (Exception ex2) {
                    if (this.debug) {
                        ex2.printStackTrace();
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        else if (this.filename != null) {
            try (final FileInputStream fileInputStream = new FileInputStream(this.filename)) {
                this.printCertFromStream(fileInputStream, printStream);
            }
        }
        else {
            this.printCertFromStream(System.in, printStream);
        }
    }
    
    private void doSelfCert(String s, String dname, String compatibleSigAlgName) throws Exception {
        if (s == null) {
            s = "mykey";
        }
        Pair<Key, char[]> recoverKey = this.recoverKey(s, this.storePass, this.keyPass);
        PrivateKey key = (PrivateKey)recoverKey.fst;
        if (this.keyPass == null) {
            this.keyPass = (char[])recoverKey.snd;
        }
        if (compatibleSigAlgName == null) {
            compatibleSigAlgName = getCompatibleSigAlgName(key.getAlgorithm());
        }
        Certificate certificate = this.keyStore.getCertificate(s);
        if (certificate == null) {
            throw new Exception(new MessageFormat(Main.rb.getString("alias.has.no.public.key")).format(new Object[] { s }));
        }
        if (!(certificate instanceof X509Certificate)) {
            throw new Exception(new MessageFormat(Main.rb.getString("alias.has.no.X.509.certificate")).format(new Object[] { s }));
        }
        X509CertInfo x509CertInfo = (X509CertInfo)new X509CertImpl(certificate.getEncoded()).get("x509.info");
        Date startDate = getStartDate(this.startDate);
        Date notAfter = new Date();
        notAfter.setTime(startDate.getTime() + this.validity * 1000L * 24L * 60L * 60L);
        x509CertInfo.set("validity", new CertificateValidity(startDate, notAfter));
        x509CertInfo.set("serialNumber", new CertificateSerialNumber(new Random().nextInt() & Integer.MAX_VALUE));
        X500Name x500Name;
        if (dname == null) {
            x500Name = (Object)x509CertInfo.get("subject.dname");
        }
        else {
            x500Name = new X500Name(dname);
            x509CertInfo.set("subject.dname", x500Name);
        }
        x509CertInfo.set("issuer.dname", x500Name);
        X509CertImpl o = new X509CertImpl(x509CertInfo);
        PSSParameterSpec defaultAlgorithmParameterSpec = AlgorithmId.getDefaultAlgorithmParameterSpec(compatibleSigAlgName, key);
        o.sign(key, defaultAlgorithmParameterSpec, compatibleSigAlgName, null);
        x509CertInfo.set("algorithmID.algorithm", (Object)o.get("x509.algorithm"));
        x509CertInfo.set("version", new CertificateVersion(2));
        x509CertInfo.set("extensions", this.createV3Extensions(null, (CertificateExtensions)x509CertInfo.get("extensions"), this.v3ext, certificate.getPublicKey(), null));
        X509CertImpl o2 = new X509CertImpl(x509CertInfo);
        o2.sign(key, defaultAlgorithmParameterSpec, compatibleSigAlgName, null);
        this.keyStore.setKeyEntry(s, key, (this.keyPass != null) ? this.keyPass : this.storePass, new Certificate[] { o2 });
        if (this.verbose) {
            System.err.println(Main.rb.getString("New.certificate.self.signed."));
            System.err.print(o2.toString());
            System.err.println();
        }
        return;
    }
    
    private boolean installReply(String s, InputStream inStream) throws Exception {
        if (s == null) {
            s = "mykey";
        }
        Pair<Key, char[]> recoverKey = this.recoverKey(s, this.storePass, this.keyPass);
        PrivateKey key = (PrivateKey)recoverKey.fst;
        if (this.keyPass == null) {
            this.keyPass = (char[])recoverKey.snd;
        }
        Certificate certificate = this.keyStore.getCertificate(s);
        if (certificate == null) {
            throw new Exception(new MessageFormat(Main.rb.getString("alias.has.no.public.key.certificate.")).format(new Object[] { s }));
        }
        Collection<Object> generateCertificates = this.cf.generateCertificates(inStream);
        if (generateCertificates.isEmpty()) {
            throw new Exception(Main.rb.getString("Reply.has.no.certificates"));
        }
        Certificate[] array = (Certificate[])generateCertificates.toArray(new Certificate[generateCertificates.size()]);
        Certificate[] chain;
        if (array.length == 1) {
            chain = this.establishCertChain(certificate, array[0]);
        }
        else {
            chain = this.validateReply(s, certificate, array);
        }
        if (chain != null) {
            this.keyStore.setKeyEntry(s, key, (this.keyPass != null) ? this.keyPass : this.storePass, chain);
            return true;
        }
        return false;
    }
    
    private boolean addTrustedCert(String alias, InputStream inStream) throws Exception {
        if (alias == null) {
            throw new Exception(Main.rb.getString("Must.specify.alias"));
        }
        if (this.keyStore.containsAlias(alias)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Certificate.not.imported.alias.alias.already.exists")).format(new Object[] { alias }));
        }
        X509Certificate cert;
        try {
            cert = (X509Certificate)this.cf.generateCertificate(inStream);
        }
        catch (ClassCastException | CertificateException ex) {
            throw new Exception(Main.rb.getString("Input.not.an.X.509.certificate"));
        }
        if (this.noprompt) {
            this.checkWeak(Main.rb.getString("the.input"), cert);
            this.keyStore.setCertificateEntry(alias, cert);
            return true;
        }
        boolean n = false;
        if (KeyStoreUtil.isSelfSigned(cert)) {
            cert.verify(cert.getPublicKey());
            n = true;
        }
        String anObject = null;
        String s = this.keyStore.getCertificateAlias(cert);
        if (s != null) {
            System.err.println(new MessageFormat(Main.rb.getString("Certificate.already.exists.in.keystore.under.alias.trustalias.")).format(new Object[] { s }));
            this.checkWeak(Main.rb.getString("the.input"), cert);
            this.printWeakWarnings(true);
            anObject = this.getYesNoReply(Main.rb.getString("Do.you.still.want.to.add.it.no."));
        }
        else if (!(n == 0)) {
            if (this.trustcacerts && this.caks != null && (s = this.caks.getCertificateAlias(cert)) != null) {
                System.err.println(new MessageFormat(Main.rb.getString("Certificate.already.exists.in.system.wide.CA.keystore.under.alias.trustalias.")).format(new Object[] { s }));
                this.checkWeak(Main.rb.getString("the.input"), cert);
                this.printWeakWarnings(true);
                anObject = this.getYesNoReply(Main.rb.getString("Do.you.still.want.to.add.it.to.your.own.keystore.no."));
            }
            if (s == null) {
                this.printX509Cert(cert, System.out);
                this.checkWeak(Main.rb.getString("the.input"), cert);
                this.printWeakWarnings(true);
                anObject = this.getYesNoReply(Main.rb.getString("Trust.this.certificate.no."));
            }
        }
        if (anObject != null) {
            if ("YES".equals(anObject)) {
                this.keyStore.setCertificateEntry(alias, cert);
                return true;
            }
            return false;
        }
        else {
            try {
                if (this.establishCertChain(null, cert) != null) {
                    this.keyStore.setCertificateEntry(alias, cert);
                    return true;
                }
            }
            catch (Exception ex2) {
                this.printX509Cert(cert, System.out);
                this.checkWeak(Main.rb.getString("the.input"), cert);
                this.printWeakWarnings(true);
                if ("YES".equals(this.getYesNoReply(Main.rb.getString("Trust.this.certificate.no.")))) {
                    this.keyStore.setCertificateEntry(alias, cert);
                    return true;
                }
                return false;
            }
            return false;
        }
    }
    
    private char[] getNewPasswd(String s, char[] a2) throws Exception {
        char[] password = null;
        for (int i = 0; i < 3; ++i) {
            System.err.print(new MessageFormat(Main.rb.getString("New.prompt.")).format(new Object[] { s }));
            char[] password2 = Password.readPassword(System.in);
            this.passwords.add(password2);
            if (password2 == null || password2.length < 6) {
                System.err.println(Main.rb.getString("Password.is.too.short.must.be.at.least.6.characters"));
            }
            else if (Arrays.equals(password2, a2)) {
                System.err.println(Main.rb.getString("Passwords.must.differ"));
            }
            else {
                System.err.print(new MessageFormat(Main.rb.getString("Re.enter.new.prompt.")).format(new Object[] { s }));
                password = Password.readPassword(System.in);
                this.passwords.add(password);
                if (!Arrays.equals(password2, password)) {
                    System.err.println(Main.rb.getString("They.don.t.match.Try.again"));
                }
                else {
                    Arrays.fill(password, ' ');
                    return password2;
                }
            }
            if (password2 != null) {
                Arrays.fill(password2, ' ');
            }
            if (password != null) {
                Arrays.fill(password, ' ');
                password = null;
            }
        }
        throw new Exception(Main.rb.getString("Too.many.failures.try.later"));
    }
    
    private String getAlias(String s) throws Exception {
        if (s != null) {
            System.err.print(new MessageFormat(Main.rb.getString("Enter.prompt.alias.name.")).format(new Object[] { s }));
        }
        else {
            System.err.print(Main.rb.getString("Enter.alias.name."));
        }
        return new BufferedReader(new InputStreamReader(System.in)).readLine();
    }
    
    private String inputStringFromStdin(String s) throws Exception {
        System.err.print(s);
        return new BufferedReader(new InputStreamReader(System.in)).readLine();
    }
    
    private char[] getKeyPasswd(String s, String s2, char[] array) throws Exception {
        int n = 0;
        char[] password;
        do {
            if (array != null) {
                System.err.println(new MessageFormat(Main.rb.getString("Enter.key.password.for.alias.")).format(new Object[] { s }));
                System.err.print(new MessageFormat(Main.rb.getString(".RETURN.if.same.as.for.otherAlias.")).format(new Object[] { s2 }));
            }
            else {
                System.err.print(new MessageFormat(Main.rb.getString("Enter.key.password.for.alias.")).format(new Object[] { s }));
            }
            System.err.flush();
            password = Password.readPassword(System.in);
            this.passwords.add(password);
            if (password == null) {
                password = array;
            }
            ++n;
        } while (password == null && n < 3);
        if (password == null) {
            throw new Exception(Main.rb.getString("Too.many.failures.try.later"));
        }
        return password;
    }
    
    private String withWeak(String s) {
        if (Main.DISABLED_CHECK.permits(Main.SIG_PRIMITIVE_SET, s, null)) {
            if (Main.LEGACY_CHECK.permits(Main.SIG_PRIMITIVE_SET, s, null)) {
                return s;
            }
            return String.format(Main.rb.getString("with.weak"), new Object[] { s });
        }
        else {
            return String.format(Main.rb.getString("with.disabled"), new Object[] { s });
        }
    }
    
    private String withWeak(PublicKey key) {
        int keySize = KeyUtil.getKeySize(key);
        if (Main.DISABLED_CHECK.permits(Main.SIG_PRIMITIVE_SET, key)) {
            if (Main.LEGACY_CHECK.permits(Main.SIG_PRIMITIVE_SET, key)) {
                if (keySize >= 0) {
                    return String.format(Main.rb.getString("key.bit"), new Object[] { Integer.valueOf(keySize), key.getAlgorithm() });
                }
                return String.format(Main.rb.getString("unknown.size.1"), new Object[] { key.getAlgorithm() });
            }
            else {
                return String.format(Main.rb.getString("key.bit.weak"), new Object[] { Integer.valueOf(keySize), key.getAlgorithm() });
            }
        }
        else {
            return String.format(Main.rb.getString("key.bit.disabled"), new Object[] { Integer.valueOf(keySize), key.getAlgorithm() });
        }
    }
    
    private void printX509Cert(X509Certificate x509Certificate, PrintStream printStream) throws Exception {
        MessageFormat messageFormat = new MessageFormat(Main.rb.getString(".PATTERN.printX509Cert.with.weak"));
        PublicKey publicKey = x509Certificate.getPublicKey();
        String s = x509Certificate.getSigAlgName();
        if (!this.isTrustedCert(x509Certificate)) {
            s = this.withWeak(s);
        }
        printStream.println(messageFormat.format(new Object[] { x509Certificate.getSubjectDN().toString(), x509Certificate.getIssuerDN().toString(), x509Certificate.getSerialNumber().toString(16), x509Certificate.getNotBefore().toString(), x509Certificate.getNotAfter().toString(), this.getCertFingerPrint("SHA-1", x509Certificate), this.getCertFingerPrint("SHA-256", x509Certificate), s, this.withWeak(publicKey), Integer.valueOf(x509Certificate.getVersion()) }));
        if ((x509Certificate instanceof X509CertImpl)) {
            CertificateExtensions certificateExtensions = (CertificateExtensions)((X509CertInfo)((X509CertImpl)x509Certificate).get("x509.info")).get("extensions");
            if (certificateExtensions != null) {
                printExtensions(Main.rb.getString("Extensions."), certificateExtensions, printStream);
            }
        }
    }
    
    private static void printExtensions(String x, CertificateExtensions certificateExtensions, PrintStream printStream) throws Exception {
        int n = 0;
        Iterator<Extension> iterator = certificateExtensions.getAllExtensions().iterator();
        Iterator<Extension> iterator2 = certificateExtensions.getUnparseableExtensions().values().iterator();
        while (iterator.hasNext() || iterator2.hasNext()) {
            Extension obj = iterator.hasNext() ? ((Extension)iterator.next()) : ((Extension)iterator2.next());
            if (n == 0) {
                printStream.println();
                printStream.println(x);
                printStream.println();
            }
            printStream.print("#" + ++n + ": " + obj);
            if (obj.getClass() == Extension.class) {
                if (obj.getExtensionValue().length == 0) {
                    printStream.println(Main.rb.getString(".Empty.value."));
                }
                else {
                    new HexDumpEncoder().encodeBuffer(obj.getExtensionValue(), printStream);
                    printStream.println();
                }
            }
            printStream.println();
        }
    }
    
    private static Pair<String, Certificate> getSigner(Certificate cert, KeyStore keyStore) throws Exception {
        if (keyStore.getCertificateAlias(cert) != null) {
            return new Pair("", cert);
        }
        Enumeration<String> aliases = keyStore.aliases();
        while (aliases.hasMoreElements()) {
            String alias = (String)aliases.nextElement();
            Certificate certificate = keyStore.getCertificate(alias);
            if (certificate != null) {
                try {
                    cert.verify(certificate.getPublicKey());
                    return new Pair(alias, certificate);
                }
                catch (Exception ex) {}
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    private X500Name getX500Name() throws IOException {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(System.in));
        String inputString = "Unknown";
        String inputString2 = "Unknown";
        String inputString3 = "Unknown";
        String inputString4 = "Unknown";
        String inputString5 = "Unknown";
        String inputString6 = "Unknown";
        int n = 20;
        while (n-- >= 0) {
            inputString = this.inputString(bufferedReader, Main.rb.getString("What.is.your.first.and.last.name."), inputString);
            inputString2 = this.inputString(bufferedReader, Main.rb.getString("What.is.the.name.of.your.organizational.unit."), inputString2);
            inputString3 = this.inputString(bufferedReader, Main.rb.getString("What.is.the.name.of.your.organization."), inputString3);
            inputString4 = this.inputString(bufferedReader, Main.rb.getString("What.is.the.name.of.your.City.or.Locality."), inputString4);
            inputString5 = this.inputString(bufferedReader, Main.rb.getString("What.is.the.name.of.your.State.or.Province."), inputString5);
            inputString6 = this.inputString(bufferedReader, Main.rb.getString("What.is.the.two.letter.country.code.for.this.unit."), inputString6);
            X500Name o = new X500Name(inputString, inputString2, inputString3, inputString4, inputString5, inputString6);
            String inputString7 = this.inputString(bufferedReader, new MessageFormat(Main.rb.getString("Is.name.correct.")).format(new Object[] { o }), Main.rb.getString("no"));
            if (Main.collator.compare(inputString7, Main.rb.getString("yes")) != 0 && Main.collator.compare(inputString7, Main.rb.getString("y")) != 0) {
                continue;
            }
            System.err.println();
            return o;
        }
        throw new RuntimeException(Main.rb.getString("Too.many.retries.program.terminated"));
    }
    
    private String inputString(BufferedReader bufferedReader, String x, String s) throws IOException {
        System.err.println(x);
        System.err.print(new MessageFormat(Main.rb.getString(".defaultValue.")).format(new Object[] { s }));
        System.err.flush();
        String line = bufferedReader.readLine();
        if (line == null || Main.collator.compare(line, "") == 0) {
            line = s;
            return line;
        }
        return line;
    }
    
    private void dumpCert(Certificate certificate, PrintStream printStream) throws IOException, CertificateException {
        if (this.rfc) {
            printStream.println("-----BEGIN CERTIFICATE-----");
            printStream.println(Base64.getMimeEncoder(64, Main.CRLF).encodeToString(certificate.getEncoded()));
            printStream.println("-----END CERTIFICATE-----");
        }
        else {
            printStream.write(certificate.getEncoded());
        }
    }
    
    private void byte2hex(byte b, StringBuffer sb) {
        char[] array = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        int n = (b & 0xF0) >> 4;
        int n2 = b & 0xF;
        sb.append(array[n]);
        sb.append(array[n2]);
    }
    
    private String toHexString(byte[] array) {
        StringBuffer sb = new StringBuffer();
        for (int length = array.length, i = 0; i < length; ++i) {
            this.byte2hex(array[i], sb);
            if (i < length - 1) {
                sb.append(":");
            }
        }
        return sb.toString();
    }
    
    private Pair<Key, char[]> recoverKey(String alias, char[] password, char[] keyPasswd) throws Exception {
        if (KeyStoreUtil.isWindowsKeyStore(this.storetype)) {
            return Pair.of(this.keyStore.getKey(alias, (char[])null), null);
        }
        if (!this.keyStore.containsAlias(alias)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { alias }));
        }
        if (!this.keyStore.entryInstanceOf(alias, KeyStore.PrivateKeyEntry.class) && !this.keyStore.entryInstanceOf(alias, KeyStore.SecretKeyEntry.class)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.has.no.key")).format(new Object[] { alias }));
        }
        Key key;
        if (keyPasswd == null) {
            try {
                key = this.keyStore.getKey(alias, password);
                keyPasswd = password;
                this.passwords.add(keyPasswd);
            }
            catch (UnrecoverableKeyException ex) {
                if (!this.token) {
                    keyPasswd = this.getKeyPasswd(alias, null, (char[])null);
                    key = this.keyStore.getKey(alias, keyPasswd);
                }
                else {
                    throw ex;
                }
            }
        }
        else {
            key = this.keyStore.getKey(alias, keyPasswd);
        }
        return Pair.of(key, keyPasswd);
    }
    
    private Pair<KeyStore.Entry, char[]> recoverEntry(KeyStore keyStore, String alias, char[] password, char[] keyPasswd) throws Exception {
        if (!keyStore.containsAlias(alias)) {
            throw new Exception(new MessageFormat(Main.rb.getString("Alias.alias.does.not.exist")).format(new Object[] { alias }));
        }
        __Null protParam = null;
        KeyStore.Entry entry;
        try {
            entry = keyStore.getEntry(alias, protParam);
            keyPasswd = null;
        }
        catch (UnrecoverableEntryException ex) {
            if ("PKCS11".equalsIgnoreCase(keyStore.getType()) || KeyStoreUtil.isWindowsKeyStore(keyStore.getType())) {
                throw ex;
            }
            if (keyPasswd != null) {
                entry = keyStore.getEntry(alias, new KeyStore.PasswordProtection(keyPasswd));
            }
            else {
                try {
                    entry = keyStore.getEntry(alias, new KeyStore.PasswordProtection(password));
                    keyPasswd = password;
                }
                catch (UnrecoverableEntryException ex2) {
                    if ("PKCS12".equalsIgnoreCase(keyStore.getType())) {
                        throw ex2;
                    }
                    keyPasswd = this.getKeyPasswd(alias, null, (char[])null);
                    entry = keyStore.getEntry(alias, new KeyStore.PasswordProtection(keyPasswd));
                }
            }
        }
        return Pair.of(entry, keyPasswd);
    }
    
    private String getCertFingerPrint(String algorithm, Certificate certificate) throws Exception {
        return this.toHexString(MessageDigest.getInstance(algorithm).digest(certificate.getEncoded()));
    }
    
    private void printNoIntegrityWarning() {
        System.err.println();
        System.err.println(Main.rb.getString(".WARNING.WARNING.WARNING."));
        System.err.println(Main.rb.getString(".The.integrity.of.the.information.stored.in.your.keystore."));
        System.err.println(Main.rb.getString(".WARNING.WARNING.WARNING."));
        System.err.println();
    }
    
    private Certificate[] validateReply(String s, Certificate certificate, Certificate[] array) throws Exception {
        this.checkWeak(Main.rb.getString("reply"), array);
        PublicKey publicKey;
        int n;
        for (publicKey = certificate.getPublicKey(), n = 0; n < array.length && !publicKey.equals(array[n].getPublicKey()); ++n) {}
        if (n == array.length) {
            throw new Exception(new MessageFormat(Main.rb.getString("Certificate.reply.does.not.contain.public.key.for.alias.")).format(new Object[] { s }));
        }
        Certificate certificate2 = array[0];
        array[0] = array[n];
        array[n] = certificate2;
        X509Certificate end = (X509Certificate)array[0];
        int i = 1;
        while (i < array.length - 1) {
            while (i < array.length) {
                if (KeyStoreUtil.signedBy(end, (X509Certificate)array[i])) {
                    Certificate certificate3 = array[i];
                    array[i] = certificate3;
                    end = (X509Certificate)array[i];
                    break;
                }
                ++i;
            }
            if (i == array.length) {
                throw new Exception(Main.rb.getString("Incomplete.certificate.chain.in.reply"));
            }
            ++i;
        }
        if (this.noprompt) {
            return array;
        }
        Certificate certificate4 = array[array.length - 1];
        boolean n2 = true;
        Pair<String, Certificate> pair = getSigner(certificate4, this.keyStore);
        if (pair == null && this.trustcacerts && this.caks != null) {
            pair = getSigner(certificate4, this.caks);
            n2 = false;
        }
        if (pair == null) {
            System.err.println();
            System.err.println(Main.rb.getString("Top.level.certificate.in.reply."));
            this.printX509Cert((X509Certificate)certificate4, System.out);
            System.err.println();
            System.err.print(Main.rb.getString(".is.not.trusted."));
            this.printWeakWarnings(true);
            if ("NO".equals(this.getYesNoReply(Main.rb.getString("Install.reply.anyway.no.")))) {
                return null;
            }
        }
        else if (pair.snd != certificate4) {
            Certificate[] array2 = new Certificate[array.length + 1];
            System.arraycopy(array, 0, array2, 0, array.length);
            array2[array2.length - 1] = (Certificate)pair.snd;
            array = array2;
            this.checkWeak(String.format(Main.rb.getString(n2 ? "alias.in.keystore" : "alias.in.cacerts"), new Object[] { pair.fst }), (Certificate)pair.snd);
        }
        return array;
    }
    
    private Certificate[] establishCertChain(Certificate other, Certificate certificate) throws Exception {
        if (other != null) {
            if (!other.getPublicKey().equals(certificate.getPublicKey())) {
                throw new Exception(Main.rb.getString("Public.keys.in.reply.and.keystore.don.t.match"));
            }
            if (certificate.equals(other)) {
                throw new Exception(Main.rb.getString("Certificate.reply.and.certificate.in.keystore.are.identical"));
            }
        }
        Object o = null;
        if (this.keyStore.size() > 0) {
            o = new Hashtable(11);
            this.keystorecerts2Hashtable(this.keyStore, o);
        }
        if (this.trustcacerts && this.caks != null && this.caks.size() > 0) {
            if (o == null) {
                o = new Hashtable(11);
            }
            this.keystorecerts2Hashtable(this.caks, o);
        }
        Vector o2 = new Vector(2);
        if (this.buildChain(new Pair(Main.rb.getString("the.input"), (Certificate)certificate), o2, o)) {
            for (Pair pair : o2) {
                this.checkWeak((String)pair.fst, (Certificate)pair.snd);
            }
            Certificate[] array = new Certificate[o2.size()];
            int n = 0;
            for (int i = o2.size() - 1; i >= 0; --i) {
                array[n] = (Certificate)((Pair)o2.elementAt(i)).snd;
                ++n;
            }
            return array;
        }
        throw new Exception(Main.rb.getString("Failed.to.establish.chain.from.reply"));
    }
    
    private boolean buildChain(Pair<String, X509Certificate> pair, Vector<Pair<String, X509Certificate>> vector, Hashtable<Principal, Vector<Pair<String, X509Certificate>>> hashtable) {
        if (KeyStoreUtil.isSelfSigned((X509Certificate)pair.snd)) {
            vector.addElement(pair);
            return true;
        }
        Vector<Object> vector2 = (Vector)hashtable.get(((X509Certificate)pair.snd).getIssuerDN());
        if (vector2 == null) {
            return false;
        }
        Enumeration<Object> elements = vector2.elements();
        while (elements.hasMoreElements()) {
            Pair<String, X509Certificate> pair2 = (Pair)elements.nextElement();
            PublicKey publicKey = ((X509Certificate)pair2.snd).getPublicKey();
            try {
                ((X509Certificate)pair.snd).verify(publicKey);
            }
            catch (Exception ex) {
                continue;
            }
            if (this.buildChain(pair2, vector, hashtable)) {
                vector.addElement(pair);
                return true;
            }
            continue;
        }
        return false;
    }
    
    private String getYesNoReply(String s) throws IOException {
        int n = 20;
        while (n-- >= 0) {
            System.err.print(s);
            System.err.flush();
            String line = new BufferedReader(new InputStreamReader(System.in)).readLine();
            String s2;
            if (Main.collator.compare(line, "") == 0 || Main.collator.compare(line, Main.rb.getString("n")) == 0 || Main.collator.compare(line, Main.rb.getString("no")) == 0) {
                s2 = "NO";
            }
            else if (Main.collator.compare(line, Main.rb.getString("y")) == 0 || Main.collator.compare(line, Main.rb.getString("yes")) == 0) {
                s2 = "YES";
            }
            else {
                System.err.println(Main.rb.getString("Wrong.answer.try.again"));
                s2 = null;
            }
            if (s2 == null) {
                continue;
            }
            return s2;
        }
        throw new RuntimeException(Main.rb.getString("Too.many.retries.program.terminated"));
    }
    
    private void keystorecerts2Hashtable(KeyStore keyStore, Hashtable<Principal, Vector<Pair<String, X509Certificate>>> hashtable) throws Exception {
        Enumeration<String> aliases = keyStore.aliases();
        while (aliases.hasMoreElements()) {
            String alias = (String)aliases.nextElement();
            Certificate certificate = keyStore.getCertificate(alias);
            if (certificate != null) {
                Principal subjectDN = ((X509Certificate)certificate).getSubjectDN();
                Pair obj = new Pair(String.format(Main.rb.getString((keyStore == this.caks) ? "alias.in.cacerts" : "alias.in.keystore"), new Object[] { alias }), (X509Certificate)certificate);
                Vector<Object> value = (Object)hashtable.get(subjectDN);
                if (value == null) {
                    value = new Vector();
                    value.addElement(obj);
                }
                else if (!value.contains(obj)) {
                    value.addElement(obj);
                }
                hashtable.put(subjectDN, value);
            }
            else {
                continue;
            }
        }
    }
    
    private static Date getStartDate(String s) throws IOException {
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        if (s != null) {
            IOException ex = new IOException(Main.rb.getString("Illegal.startdate.value"));
            int length = s.length();
            if (length == 0) {
                throw ex;
            }
            if (s.charAt(0) == '-' || s.charAt(0) == '+') {
                int i = 0;
                while (i < length) {
                    int n;
                    switch (s.charAt(i)) {
                        case '+': {
                            n = 1;
                            break;
                        }
                        case '-': {
                            n = -1;
                            break;
                        }
                        default: {
                            throw ex;
                        }
                    }
                    int j = i + 1;
                    while (j < length) {
                        int char1 = s.charAt(j);
                        if (char1 >= '0') {
                            if (char1 > '9') {
                                break;
                            }
                            ++j;
                        }
                        else {
                            break;
                        }
                    }
                    if (j == i + 1) {
                        throw ex;
                    }
                    int int1 = Integer.parseInt(s.substring(i + 1, j));
                    if (j >= length) {
                        throw ex;
                    }
                    int n2;
                    switch (s.charAt(j)) {
                        case 'y': {
                            n2 = 1;
                            break;
                        }
                        case 'm': {
                            n2 = 2;
                            break;
                        }
                        case 'd': {
                            n2 = 5;
                            break;
                        }
                        case 'H': {
                            n2 = 10;
                            break;
                        }
                        case 'M': {
                            n2 = 12;
                            break;
                        }
                        case 'S': {
                            n2 = 13;
                            break;
                        }
                        default: {
                            throw ex;
                        }
                    }
                    gregorianCalendar.add(n2, n * int1);
                    i = j + 1;
                }
            }
            else {
                String substring = null;
                String substring2 = null;
                if (length == 19) {
                    substring = s.substring(0, 10);
                    substring2 = s.substring(11);
                    if (s.charAt(10) != ' ') {
                        throw ex;
                    }
                }
                else if (length == 10) {
                    substring = s;
                }
                else if (length == 8) {
                    substring2 = s;
                }
                else {
                    throw ex;
                }
                if (substring != null) {
                    if (substring.matches("\\d\\d\\d\\d\\/\\d\\d\\/\\d\\d")) {
                        gregorianCalendar.set((int)Integer.valueOf(substring.substring(0, 4)), (int)Integer.valueOf(substring.substring(5, 7)) - 1, (int)Integer.valueOf(substring.substring(8, 10)));
                    }
                    else {
                        throw ex;
                    }
                }
                if (substring2 != null) {
                    if (substring2.matches("\\d\\d:\\d\\d:\\d\\d")) {
                        gregorianCalendar.set(11, (int)Integer.valueOf(substring2.substring(0, 2)));
                        gregorianCalendar.set(12, (int)Integer.valueOf(substring2.substring(0, 2)));
                        gregorianCalendar.set(13, (int)Integer.valueOf(substring2.substring(0, 2)));
                        gregorianCalendar.set(14, 0);
                    }
                    else {
                        throw ex;
                    }
                }
            }
        }
        return gregorianCalendar.getTime();
    }
    
    private static int oneOf(String anotherString, String... array) throws Exception {
        int[] array2 = new int[array.length];
        int n = 0;
        int n2 = Integer.MAX_VALUE;
        for (int i = 0; i < array.length; ++i) {
            String s = array[i];
            if (s == null) {
                n2 = i;
            }
            else if (s.toLowerCase(Locale.ENGLISH).startsWith(anotherString.toLowerCase(Locale.ENGLISH))) {
                array2[n++] = i;
            }
            else {
                StringBuffer o = new StringBuffer();
                int n3 = 1;
                for (int c : s.toCharArray()) {
                    if (n3 != 0) {
                        o.append((char)c);
                        n3 = 0;
                    }
                    else if (!Character.isLowerCase((char)c)) {
                        o.append((char)c);
                    }
                }
                if (o.toString().equalsIgnoreCase(anotherString)) {
                    array2[n++] = i;
                }
            }
        }
        if (n == 0) {
            return -1;
        }
        if (n == 1) {
            return array2[0];
        }
        if (array2[1] > n2) {
            return array2[0];
        }
        StringBuffer o2 = new StringBuffer();
        o2.append(new MessageFormat(Main.rb.getString("command.{0}.is.ambiguous.")).format(new Object[] { anotherString }));
        o2.append("\n    ");
        for (int n4 = 0; n4 < n && array2[n4] < n2; ++n4) {
            o2.append(' ');
            o2.append(array[array2[n4]]);
        }
        throw new Exception(o2.toString());
    }
    
    private GeneralName createGeneralName(String str, String name) throws Exception {
        int one = oneOf(str, new String[] { "EMAIL", "URI", "DNS", "IP", "OID" });
        if (one < 0) {
            throw new Exception(Main.rb.getString("Unrecognized.GeneralName.type.") + str);
        }
        RFC822Name name2;
        switch (one) {
            case 0: {
                name2 = new RFC822Name(name);
                break;
            }
            case 1: {
                name2 = new URIName(name);
                break;
            }
            case 2: {
                name2 = new DNSName(name, true);
                break;
            }
            case 3: {
                name2 = new IPAddressName(name);
                break;
            }
            default: {
                name2 = new OIDName(name);
                break;
            }
        }
        return new GeneralName(name2);
    }
    
    private ObjectIdentifier findOidForExtName(String oid) throws Exception {
        switch (oneOf(oid, Main.extSupported)) {
            case 0: {
                return PKIXExtensions.BasicConstraints_Id;
            }
            case 1: {
                return PKIXExtensions.KeyUsage_Id;
            }
            case 2: {
                return PKIXExtensions.ExtendedKeyUsage_Id;
            }
            case 3: {
                return PKIXExtensions.SubjectAlternativeName_Id;
            }
            case 4: {
                return PKIXExtensions.IssuerAlternativeName_Id;
            }
            case 5: {
                return PKIXExtensions.SubjectInfoAccess_Id;
            }
            case 6: {
                return PKIXExtensions.AuthInfoAccess_Id;
            }
            case 8: {
                return PKIXExtensions.CRLDistributionPoints_Id;
            }
            default: {
                return new ObjectIdentifier(oid);
            }
        }
    }
    
    private CertificateExtensions createV3Extensions(CertificateExtensions certificateExtensions, CertificateExtensions certificateExtensions2, List<String> list, PublicKey pubKey, PublicKey publicKey) throws Exception {
        if (certificateExtensions2 != null && certificateExtensions != null) {
            throw new Exception("One of request and original should be null.");
        }
        if (certificateExtensions2 == null) {
            certificateExtensions2 = new CertificateExtensions();
        }
        try {
            if (certificateExtensions != null) {
                for (String s : list) {
                    if (s.toLowerCase(Locale.ENGLISH).startsWith("honored=")) {
                        List<String> list2 = Arrays.asList(s.toLowerCase(Locale.ENGLISH).substring(8).split(","));
                        if (list2.contains("all")) {
                            certificateExtensions2 = certificateExtensions;
                        }
                        for (String str : list2) {
                            if (str.equals("all")) {
                                continue;
                            }
                            boolean n = true;
                            int one = -1;
                            String s2 = null;
                            if (str.startsWith("-")) {
                                n = false;
                                s2 = str.substring(1);
                            }
                            else {
                                int index = str.indexOf(58);
                                if (index >= 0) {
                                    s2 = str.substring(0, index);
                                    one = oneOf(str.substring(index + 1), new String[] { "critical", "non-critical" });
                                    if (one == -1) {
                                        throw new Exception(Main.rb.getString("Illegal.value.") + str);
                                    }
                                }
                            }
                            String nameByOid = certificateExtensions.getNameByOid(this.findOidForExtName(s2));
                            if (!(n == 0)) {
                                Extension value = certificateExtensions.get(nameByOid);
                                if ((!value.isCritical() && one == 0) || (value.isCritical() && one == 1)) {
                                    certificateExtensions2.set(nameByOid, Extension.newExtension(value.getExtensionId(), !value.isCritical(), value.getExtensionValue()));
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                certificateExtensions2.delete(nameByOid);
                            }
                        }
                        break;
                    }
                    else {
                        continue;
                    }
                }
            }
            for (String str2 : list) {
                boolean critical = false;
                int index2 = str2.indexOf(61);
                String oid;
                String substring;
                if (index2 >= 0) {
                    oid = str2.substring(0, index2);
                    substring = str2.substring(index2 + 1);
                }
                else {
                    oid = str2;
                    substring = null;
                }
                int index3 = oid.indexOf(58);
                if (index3 >= 0) {
                    if (oneOf(oid.substring(index3 + 1), new String[] { "critical" }) == 0) {
                        critical = true;
                    }
                    oid = oid.substring(0, index3);
                }
                if (oid.equalsIgnoreCase("honored")) {
                    continue;
                }
                int one2 = oneOf(oid, Main.extSupported);
                switch (one2) {
                    case 0: {
                        int len = -1;
                        boolean boolean1 = false;
                        if (substring == null) {
                            boolean1 = true;
                        }
                        else {
                            try {
                                len = Integer.parseInt(substring);
                                boolean1 = true;
                            }
                            catch (NumberFormatException ex) {
                                String[] split = substring.split(",");
                                int length = split.length;
                                int i = 0;
                                while (i < length) {
                                    String[] split2 = split[i].split(":");
                                    if (split2.length != 2) {
                                        throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                                    }
                                    if (split2[0].equalsIgnoreCase("ca")) {
                                        boolean1 = Boolean.parseBoolean(split2[1]);
                                    }
                                    else if (split2[0].equalsIgnoreCase("pathlen")) {
                                        len = Integer.parseInt(split2[1]);
                                    }
                                    else {
                                        throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                                    }
                                    ++i;
                                }
                            }
                        }
                        certificateExtensions2.set("BasicConstraints", new BasicConstraintsExtension(Boolean.valueOf((boolean)critical), (boolean)boolean1, len));
                        continue;
                    }
                    case 1: {
                        if (substring != null) {
                            boolean[] bitString = new boolean[9];
                            String[] split3 = substring.split(",");
                            int length2 = split3.length;
                            int j = 0;
                            while (j < length2) {
                                String str3 = split3[j];
                                int one3 = oneOf(str3, new String[] { "digitalSignature", "nonRepudiation", "keyEncipherment", "dataEncipherment", "keyAgreement", "keyCertSign", "cRLSign", "encipherOnly", "decipherOnly", "contentCommitment" });
                                if (one3 < 0) {
                                    throw new Exception(Main.rb.getString("Unknown.keyUsage.type.") + str3);
                                }
                                if (one3 == 9) {
                                    one3 = 1;
                                }
                                bitString[one3] = true;
                                ++j;
                            }
                            KeyUsageExtension o = new KeyUsageExtension(bitString);
                            certificateExtensions2.set("KeyUsage", Extension.newExtension(o.getExtensionId(), (boolean)critical, o.getExtensionValue()));
                            continue;
                        }
                        throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                    }
                    case 2: {
                        if (substring != null) {
                            Vector keyUsages = new Vector();
                            for (String s3 : substring.split(",")) {
                                int one4 = oneOf(s3, new String[] { "anyExtendedKeyUsage", "serverAuth", "clientAuth", "codeSigning", "emailProtection", "", "", "", "timeStamping", "OCSPSigning" });
                                if (one4 < 0) {
                                    try {
                                        keyUsages.add(new ObjectIdentifier(s3));
                                    }
                                    catch (Exception ex2) {
                                        throw new Exception(Main.rb.getString("Unknown.extendedkeyUsage.type.") + s3);
                                    }
                                }
                                else if (one4 == 0) {
                                    keyUsages.add(new ObjectIdentifier("2.5.29.37.0"));
                                }
                                else {
                                    keyUsages.add(new ObjectIdentifier("1.3.6.1.5.5.7.3." + one4));
                                }
                            }
                            certificateExtensions2.set("ExtendedKeyUsage", new ExtendedKeyUsageExtension(Boolean.valueOf((boolean)critical), keyUsages));
                            continue;
                        }
                        throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                    }
                    case 3:
                    case 4: {
                        if (substring != null) {
                            String[] split5 = substring.split(",");
                            GeneralNames o2 = new GeneralNames();
                            int length4 = split5.length;
                            int l = 0;
                            while (l < length4) {
                                String str4 = split5[l];
                                int index4 = str4.indexOf(58);
                                if (index4 < 0) {
                                    throw new Exception("Illegal item " + str4 + " in " + str2);
                                }
                                o2.add(this.createGeneralName(str4.substring(0, index4), str4.substring(index4 + 1)));
                                ++l;
                            }
                            if (one2 == 3) {
                                certificateExtensions2.set("SubjectAlternativeName", new SubjectAlternativeNameExtension(Boolean.valueOf((boolean)critical), o2));
                            }
                            else {
                                certificateExtensions2.set("IssuerAlternativeName", new IssuerAlternativeNameExtension(Boolean.valueOf((boolean)critical), o2));
                            }
                            continue;
                        }
                        else {
                            throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                        }
                    }
                    case 5:
                    case 6: {
                        if (critical != 0) {
                            throw new Exception(Main.rb.getString("This.extension.cannot.be.marked.as.critical.") + str2);
                        }
                        if (substring != null) {
                            ArrayList o3 = new ArrayList();
                            String[] split6 = substring.split(",");
                            int length5 = split6.length;
                            int n2 = 0;
                            while (n2 < length5) {
                                String s4 = split6[n2];
                                int index5 = s4.indexOf(58);
                                int index6 = s4.indexOf(58, index5 + 1);
                                if (index5 < 0 || index6 < 0) {
                                    throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                                }
                                String substring2 = s4.substring(0, index5);
                                String substring3 = s4.substring(index5 + 1, index6);
                                String substring4 = s4.substring(index6 + 1);
                                int one5 = oneOf(substring2, new String[] { "", "ocsp", "caIssuers", "timeStamping", "", "caRepository" });
                                Object accessMethod;
                                if (one5 < 0) {
                                    try {
                                        accessMethod = new ObjectIdentifier(substring2);
                                    }
                                    catch (Exception ex3) {
                                        throw new Exception(Main.rb.getString("Unknown.AccessDescription.type.") + substring2);
                                    }
                                }
                                else {
                                    accessMethod = new ObjectIdentifier("1.3.6.1.5.5.7.48." + one5);
                                }
                                o3.add(new AccessDescription(accessMethod, this.createGeneralName(substring3, substring4)));
                                ++n2;
                            }
                            if (one2 == 5) {
                                certificateExtensions2.set("SubjectInfoAccess", new SubjectInfoAccessExtension(o3));
                            }
                            else {
                                certificateExtensions2.set("AuthorityInfoAccess", new AuthorityInfoAccessExtension(o3));
                            }
                            continue;
                        }
                        else {
                            throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                        }
                    }
                    case 8: {
                        if (substring != null) {
                            String[] split7 = substring.split(",");
                            GeneralNames fullName = new GeneralNames();
                            int length6 = split7.length;
                            int n3 = 0;
                            while (n3 < length6) {
                                String str5 = split7[n3];
                                int index7 = str5.indexOf(58);
                                if (index7 < 0) {
                                    throw new Exception("Illegal item " + str5 + " in " + str2);
                                }
                                fullName.add(this.createGeneralName(str5.substring(0, index7), str5.substring(index7 + 1)));
                                ++n3;
                            }
                            certificateExtensions2.set("CRLDistributionPoints", new CRLDistributionPointsExtension((boolean)critical, Collections.singletonList(new DistributionPoint(fullName, null, null))));
                            continue;
                        }
                        throw new Exception(Main.rb.getString("Illegal.value.") + str2);
                    }
                    case -1: {
                        ObjectIdentifier extensionId = new ObjectIdentifier(oid);
                        byte[] copy;
                        if (substring != null) {
                            byte[] original = new byte[substring.length() / 2 + 1];
                            int n4 = 0;
                            for (int n6 : substring.toCharArray()) {
                                int n7;
                                if (n6 >= '0' && n6 <= '9') {
                                    n7 = n6 - 48;
                                }
                                else if (n6 >= 'A' && n6 <= 'F') {
                                    n7 = n6 - 65 + 10;
                                }
                                else if (n6 >= 'a' && n6 <= 'f') {
                                    n7 = n6 - 97 + 10;
                                }
                                else {
                                    continue;
                                }
                                if (n4 % 2 == 0) {
                                    original[n4 / 2] = (byte)(n7 << 4);
                                }
                                else {
                                    int n8 = n4 / 2;
                                    original[n8] += n7;
                                }
                                ++n4;
                            }
                            if (n4 % 2 != 0) {
                                throw new Exception(Main.rb.getString("Odd.number.of.hex.digits.found.") + str2);
                            }
                            copy = Arrays.copyOf(original, n4 / 2);
                        }
                        else {
                            copy = new byte[0];
                        }
                        certificateExtensions2.set(extensionId.toString(), new Extension(extensionId, (boolean)critical, new DerValue((byte)4, copy).toByteArray()));
                        continue;
                    }
                    default: {
                        throw new Exception(Main.rb.getString("Unknown.extension.type.") + str2);
                    }
                }
            }
            certificateExtensions2.set("SubjectKeyIdentifier", new SubjectKeyIdentifierExtension(new KeyIdentifier(pubKey).getIdentifier()));
            if (publicKey != null && !pubKey.equals(publicKey)) {
                certificateExtensions2.set("AuthorityKeyIdentifier", new AuthorityKeyIdentifierExtension(new KeyIdentifier(publicKey), null, null));
            }
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
        return certificateExtensions2;
    }
    
    private boolean isTrustedCert(Certificate certificate) throws KeyStoreException {
        if (this.caks != null && this.caks.getCertificateAlias(certificate) != null) {
            return true;
        }
        String certificateAlias = this.keyStore.getCertificateAlias(certificate);
        return certificateAlias != null && this.keyStore.isCertificateEntry(certificateAlias);
    }
    
    private void checkWeak(String s, String s2, Key key) {
        if (s2 != null) {
            if (!Main.DISABLED_CHECK.permits(Main.SIG_PRIMITIVE_SET, s2, null)) {
                this.weakWarnings.add(String.format(Main.rb.getString("whose.sigalg.disabled"), new Object[] { s, s2 }));
            }
            else if (!Main.LEGACY_CHECK.permits(Main.SIG_PRIMITIVE_SET, s2, null)) {
                this.weakWarnings.add(String.format(Main.rb.getString("whose.sigalg.weak"), new Object[] { s, s2 }));
            }
        }
        if (key != null) {
            if (!Main.DISABLED_CHECK.permits(Main.SIG_PRIMITIVE_SET, key)) {
                this.weakWarnings.add(String.format(Main.rb.getString("whose.key.disabled"), new Object[] { s, String.format(Main.rb.getString("key.bit"), new Object[] { Integer.valueOf(KeyUtil.getKeySize(key)), this.fullDisplayAlgName(key) }) }));
            }
            else if (!Main.LEGACY_CHECK.permits(Main.SIG_PRIMITIVE_SET, key)) {
                this.weakWarnings.add(String.format(Main.rb.getString("whose.key.weak"), new Object[] { s, String.format(Main.rb.getString("key.bit"), new Object[] { Integer.valueOf(KeyUtil.getKeySize(key)), this.fullDisplayAlgName(key) }) }));
            }
        }
    }
    
    private String fullDisplayAlgName(Key key) {
        String str = key.getAlgorithm();
        if ((key instanceof ECKey)) {
            ECParameterSpec params = ((ECKey)key).getParams();
            if ((params instanceof NamedCurve)) {
                str = str + " (" + params.toString().split(" ")[0] + ")";
            }
        }
        return str;
    }
    
    private void checkWeak(String oneInMany, Certificate[] array) throws KeyStoreException {
        for (int i = 0; i < array.length; ++i) {
            Certificate certificate = array[i];
            if ((certificate instanceof X509Certificate)) {
                X509Certificate x509Certificate = (X509Certificate)certificate;
                if (array.length > 1) {
                    oneInMany = oneInMany(oneInMany, i, array.length);
                }
                this.checkWeak(oneInMany, x509Certificate);
            }
        }
    }
    
    private void checkWeak(String s, Certificate certificate) throws KeyStoreException {
        if ((certificate instanceof X509Certificate)) {
            X509Certificate x509Certificate = (X509Certificate)certificate;
            this.checkWeak(s, this.isTrustedCert(certificate) ? null : x509Certificate.getSigAlgName(), x509Certificate.getPublicKey());
        }
    }
    
    private void checkWeak(String s, PKCS10 pkcs10) {
        this.checkWeak(s, pkcs10.getSigAlg(), pkcs10.getSubjectPublicKeyInfo());
    }
    
    private void checkWeak(String s, CRL crl, Key key) {
        if ((crl instanceof X509CRLImpl)) {
            this.checkWeak(s, ((X509CRLImpl)crl).getSigAlgName(), key);
        }
    }
    
    private void printWeakWarnings(boolean b) {
        if (!this.weakWarnings.isEmpty() && !this.nowarn) {
            System.err.println("\nWarning:");
            Iterator<String> iterator = this.weakWarnings.iterator();
            while (iterator.hasNext()) {
                System.err.println((String)iterator.next());
            }
            if (b) {
                System.err.println();
            }
        }
        this.weakWarnings.clear();
    }
    
    private void usage() {
        if (this.command != null) {
            System.err.println("keytool " + this.command + Main.rb.getString(".OPTION."));
            System.err.println();
            System.err.println(Main.rb.getString(this.command.description));
            System.err.println();
            System.err.println(Main.rb.getString("Options."));
            System.err.println();
            String[] array = new String[this.command.options.length];
            String[] array2 = new String[this.command.options.length];
            int length2 = 0;
            Main.Option option;
            for (int j = 0; j < array.length; array2[j] = Main.rb.getString(option.description), ++j) {
                option = this.command.options[j];
                array[j] = option.toString();
                if (option.arg != null) {
                    StringBuilder sb = new StringBuilder();
                    int n = j;
                    array[n] = sb.append(array[n]).append(" ").append(option.arg).toString();
                }
                if (array[j].length() > length2) {
                    length2 = array[j].length();
                }
            }
            for (int k = 0; k < array.length; ++k) {
                System.err.printf(" %-" + length2 + "s  %s\n", new Object[] { array[k], array2[k] });
            }
            System.err.println();
            System.err.println(Main.rb.getString("Use.keytool.help.for.all.available.commands"));
        }
        else {
            System.err.println(Main.rb.getString("Key.and.Certificate.Management.Tool"));
            System.err.println();
            System.err.println(Main.rb.getString("Commands."));
            System.err.println();
            Main.Command[] values = Main.Command.values();
            int length = values.length;
            int i = 0;
            while (i < length) {
                Main.Command command = values[i];
                if (command == Main.Command.KEYCLONE) {
                    break;
                }
                System.err.printf(" %-20s%s\n", new Object[] { command, Main.rb.getString(command.description) });
                ++i;
            }
            System.err.println();
            System.err.println(Main.rb.getString("Use.keytool.command.name.help.for.usage.of.command.name"));
        }
    }
    
    private void tinyHelp() {
        this.usage();
        if (this.debug) {
            throw new RuntimeException("NO BIG ERROR, SORRY");
        }
        System.exit(1);
        return;
    }
    
    private void errorNeedArgument(String s) {
        System.err.println(new MessageFormat(Main.rb.getString("Command.option.flag.needs.an.argument.")).format(new Object[] { s }));
        this.tinyHelp();
    }
    
    private char[] getPass(String s, String s2) {
        char[] passWithModifier = KeyStoreUtil.getPassWithModifier(s, s2, Main.rb);
        if (passWithModifier != null) {
            return passWithModifier;
        }
        this.tinyHelp();
        return null;
    }
    
    static {
        CRLF = new byte[] { 13, 10 };
        DISABLED_CHECK = new DisabledAlgorithmConstraints("jdk.certpath.disabledAlgorithms");
        LEGACY_CHECK = new DisabledAlgorithmConstraints("jdk.security.legacyAlgorithms");
        SIG_PRIMITIVE_SET = Collections.unmodifiableSet(EnumSet.of(CryptoPrimitive.SIGNATURE));
        PARAM_STRING = new Class[] { String.class };
        rb = ResourceBundle.getBundle("sun.security.tools.keytool.Resources");
        (collator = Collator.getInstance()).setStrength(0);
        extSupported = new String[] { "BasicConstraints", "KeyUsage", "ExtendedKeyUsage", "SubjectAlternativeName", "IssuerAlternativeName", "SubjectInfoAccess", "AuthorityInfoAccess", null, "CRLDistributionPoints" };
    }
    
    enum Command
    {
        CERTREQ("Generates.a.certificate.request", new Main.Option[] { Main.Option.ALIAS, Main.Option.SIGALG, Main.Option.FILEOUT, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.DNAME, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        CHANGEALIAS("Changes.an.entry.s.alias", new Main.Option[] { Main.Option.ALIAS, Main.Option.DESTALIAS, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        DELETE("Deletes.an.entry", new Main.Option[] { Main.Option.ALIAS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        EXPORTCERT("Exports.certificate", new Main.Option[] { Main.Option.RFC, Main.Option.ALIAS, Main.Option.FILEOUT, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        GENKEYPAIR("Generates.a.key.pair", new Main.Option[] { Main.Option.ALIAS, Main.Option.KEYALG, Main.Option.KEYSIZE, Main.Option.SIGALG, Main.Option.DESTALIAS, Main.Option.DNAME, Main.Option.STARTDATE, Main.Option.EXT, Main.Option.VALIDITY, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        GENSECKEY("Generates.a.secret.key", new Main.Option[] { Main.Option.ALIAS, Main.Option.KEYPASS, Main.Option.KEYALG, Main.Option.KEYSIZE, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        GENCERT("Generates.certificate.from.a.certificate.request", new Main.Option[] { Main.Option.RFC, Main.Option.INFILE, Main.Option.OUTFILE, Main.Option.ALIAS, Main.Option.SIGALG, Main.Option.DNAME, Main.Option.STARTDATE, Main.Option.EXT, Main.Option.VALIDITY, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        IMPORTCERT("Imports.a.certificate.or.a.certificate.chain", new Main.Option[] { Main.Option.NOPROMPT, Main.Option.TRUSTCACERTS, Main.Option.PROTECTED, Main.Option.ALIAS, Main.Option.FILEIN, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        IMPORTPASS("Imports.a.password", new Main.Option[] { Main.Option.ALIAS, Main.Option.KEYPASS, Main.Option.KEYALG, Main.Option.KEYSIZE, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        IMPORTKEYSTORE("Imports.one.or.all.entries.from.another.keystore", new Main.Option[] { Main.Option.SRCKEYSTORE, Main.Option.DESTKEYSTORE, Main.Option.SRCSTORETYPE, Main.Option.DESTSTORETYPE, Main.Option.SRCSTOREPASS, Main.Option.DESTSTOREPASS, Main.Option.SRCPROTECTED, Main.Option.SRCPROVIDERNAME, Main.Option.DESTPROVIDERNAME, Main.Option.SRCALIAS, Main.Option.DESTALIAS, Main.Option.SRCKEYPASS, Main.Option.DESTKEYPASS, Main.Option.NOPROMPT, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        KEYPASSWD("Changes.the.key.password.of.an.entry", new Main.Option[] { Main.Option.ALIAS, Main.Option.KEYPASS, Main.Option.NEW, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        LIST("Lists.entries.in.a.keystore", new Main.Option[] { Main.Option.RFC, Main.Option.ALIAS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        PRINTCERT("Prints.the.content.of.a.certificate", new Main.Option[] { Main.Option.RFC, Main.Option.FILEIN, Main.Option.SSLSERVER, Main.Option.JARFILE, Main.Option.V }), 
        PRINTCERTREQ("Prints.the.content.of.a.certificate.request", new Main.Option[] { Main.Option.FILEIN, Main.Option.V }), 
        PRINTCRL("Prints.the.content.of.a.CRL.file", new Main.Option[] { Main.Option.FILEIN, Main.Option.V }), 
        STOREPASSWD("Changes.the.store.password.of.a.keystore", new Main.Option[] { Main.Option.NEW, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        KEYCLONE("Clones.a.key.entry", new Main.Option[] { Main.Option.ALIAS, Main.Option.DESTALIAS, Main.Option.KEYPASS, Main.Option.NEW, Main.Option.STORETYPE, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        SELFCERT("Generates.a.self.signed.certificate", new Main.Option[] { Main.Option.ALIAS, Main.Option.SIGALG, Main.Option.DNAME, Main.Option.STARTDATE, Main.Option.VALIDITY, Main.Option.KEYPASS, Main.Option.STORETYPE, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V }), 
        GENCRL("Generates.CRL", new Main.Option[] { Main.Option.RFC, Main.Option.FILEOUT, Main.Option.ID, Main.Option.ALIAS, Main.Option.SIGALG, Main.Option.EXT, Main.Option.KEYPASS, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.STORETYPE, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V, Main.Option.PROTECTED }), 
        IDENTITYDB("Imports.entries.from.a.JDK.1.1.x.style.identity.database", new Main.Option[] { Main.Option.FILEIN, Main.Option.STORETYPE, Main.Option.KEYSTORE, Main.Option.STOREPASS, Main.Option.PROVIDERNAME, Main.Option.PROVIDERCLASS, Main.Option.PROVIDERARG, Main.Option.PROVIDERPATH, Main.Option.V });
        
        final String description;
        final Main.Option[] options;
        
        private Command(String description, Main.Option[] options) {
            this.description = description;
            this.options = options;
        }
        
        @Override
        public String toString() {
            return "-" + this.name().toLowerCase(Locale.ENGLISH);
        }
    }
    
    enum Option
    {
        ALIAS("alias", "<alias>", "alias.name.of.the.entry.to.process"), 
        DESTALIAS("destalias", "<destalias>", "destination.alias"), 
        DESTKEYPASS("destkeypass", "<arg>", "destination.key.password"), 
        DESTKEYSTORE("destkeystore", "<destkeystore>", "destination.keystore.name"), 
        DESTPROTECTED("destprotected", null, "destination.keystore.password.protected"), 
        DESTPROVIDERNAME("destprovidername", "<destprovidername>", "destination.keystore.provider.name"), 
        DESTSTOREPASS("deststorepass", "<arg>", "destination.keystore.password"), 
        DESTSTORETYPE("deststoretype", "<deststoretype>", "destination.keystore.type"), 
        DNAME("dname", "<dname>", "distinguished.name"), 
        EXT("ext", "<value>", "X.509.extension"), 
        FILEOUT("file", "<filename>", "output.file.name"), 
        FILEIN("file", "<filename>", "input.file.name"), 
        ID("id", "<id:reason>", "Serial.ID.of.cert.to.revoke"), 
        INFILE("infile", "<filename>", "input.file.name"), 
        KEYALG("keyalg", "<keyalg>", "key.algorithm.name"), 
        KEYPASS("keypass", "<arg>", "key.password"), 
        KEYSIZE("keysize", "<keysize>", "key.bit.size"), 
        KEYSTORE("keystore", "<keystore>", "keystore.name"), 
        NEW("new", "<arg>", "new.password"), 
        NOPROMPT("noprompt", null, "do.not.prompt"), 
        OUTFILE("outfile", "<filename>", "output.file.name"), 
        PROTECTED("protected", null, "password.through.protected.mechanism"), 
        PROVIDERARG("providerarg", "<arg>", "provider.argument"), 
        PROVIDERCLASS("providerclass", "<providerclass>", "provider.class.name"), 
        PROVIDERNAME("providername", "<providername>", "provider.name"), 
        PROVIDERPATH("providerpath", "<pathlist>", "provider.classpath"), 
        RFC("rfc", null, "output.in.RFC.style"), 
        SIGALG("sigalg", "<sigalg>", "signature.algorithm.name"), 
        SRCALIAS("srcalias", "<srcalias>", "source.alias"), 
        SRCKEYPASS("srckeypass", "<arg>", "source.key.password"), 
        SRCKEYSTORE("srckeystore", "<srckeystore>", "source.keystore.name"), 
        SRCPROTECTED("srcprotected", null, "source.keystore.password.protected"), 
        SRCPROVIDERNAME("srcprovidername", "<srcprovidername>", "source.keystore.provider.name"), 
        SRCSTOREPASS("srcstorepass", "<arg>", "source.keystore.password"), 
        SRCSTORETYPE("srcstoretype", "<srcstoretype>", "source.keystore.type"), 
        SSLSERVER("sslserver", "<server[:port]>", "SSL.server.host.and.port"), 
        JARFILE("jarfile", "<filename>", "signed.jar.file"), 
        STARTDATE("startdate", "<startdate>", "certificate.validity.start.date.time"), 
        STOREPASS("storepass", "<arg>", "keystore.password"), 
        STORETYPE("storetype", "<storetype>", "keystore.type"), 
        TRUSTCACERTS("trustcacerts", null, "trust.certificates.from.cacerts"), 
        V("v", null, "verbose.output"), 
        VALIDITY("validity", "<valDays>", "validity.number.of.days");
        
        final String name;
        final String arg;
        final String description;
        
        private Option(String name2, String arg, String description) {
            this.name = name2;
            this.arg = arg;
            this.description = description;
        }
        
        @Override
        public String toString() {
            return "-" + this.name;
        }
    }
}
